"""Base class for Kilo Code configuration builders.

This module provides the KiloCodeBuilder base class that contains common
functionality shared between CLI and IDE targets for Kilo Code.

Classes:
    KiloCodeBuilder: Base builder for Kilo Code configurations.

Example:
    >>> from promptosaurus.builders.kilo.kilo_code_builder import KiloCodeBuilder
    >>> # Subclass to create a specific builder
    >>> class MyBuilder(KiloCodeBuilder):
    ...     def build(self, output, config=None, dry_run=False):
    ...         return ["Created files"]
"""

import shutil
from pathlib import Path
from typing import Any

from promptosaurus.builders.builder import Builder
from promptosaurus.builders.config import KiloConfig
from promptosaurus.builders.ignore_generator import KiloIgnoreBuilder
from promptosaurus.builders.utils import HeaderStripper
from promptosaurus.registry import registry


class KiloCodeBuilder(Builder):
    """Base builder for Kilo Code configurations.

    This abstract class provides common functionality for both CLI and IDE
    output formats. It handles:
    - Configuration management via KiloConfig
    - File copying with template variable substitution
    - Manifest file generation
    - Ignore file generation
    - Base and mode file creation

    Attributes:
        kilo_modes: Property returning the kilo modes from config.
        language_file_map: Property returning the language file map from config.
        _kilo_builtin_modes: Frozenset of built-in Kilo mode names.

    Args:
        config: Optional KiloConfig instance. If not provided, uses default config.

    Example:
        >>> builder = KiloCodeBuilder()
        >>> modes = builder.kilo_modes
        >>> print(len(modes) > 0)
        True
    """

    # Modes that are built-in to Kilo and should not be generated in output
    _kilo_builtin_modes = frozenset(
        {
            "architect",
            "code",
            "ask",
            "debug",
            "orchestrator",
        }
    )

    def __init__(self, config: KiloConfig | None = None) -> None:
        """Initialize builder with optional config.

        Args:
            config: KiloConfig instance. Uses default if not provided.

        Example:
            >>> # Use default config
            >>> builder = KiloCodeBuilder()
            >>> # Use custom config
            >>> custom_config = KiloConfig()
            >>> builder = KiloCodeBuilder(config=custom_config)
        """
        self._config = config or KiloConfig()

    @property
    def kilo_modes(self) -> dict[str, Any]:
        """Return the kilo modes from config.

        Returns:
            Dictionary of mode slug to mode configuration.

        Example:
            >>> builder = KiloCodeBuilder()
            >>> modes = builder.kilo_modes
            >>> print("code" in modes)
            True
        """
        return self._config.kilo_modes

    @property
    def language_file_map(self) -> dict[str, str]:
        """Return the language file map from config.

        Returns:
            Dictionary mapping language name to conventions file.

        Example:
            >>> builder = KiloCodeBuilder()
            >>> lang_map = builder.language_file_map
            >>> print(lang_map.get("python", ""))
            core-conventions-python.md
        """
        return self._config.language_file_map

    def build(
        self, output: Path, config: dict[str, Any] | None = None, dry_run: bool = False
    ) -> list[str]:
        """Build Kilo Code configuration. Subclasses implement specific output formats.

        This method must be implemented by subclasses to generate the
        appropriate output format for their target.

        Args:
            output: Directory path where files will be created.
            config: Optional configuration dict with template variables.
            dry_run: If True, preview what would be written without touching filesystem.

        Returns:
            List of action strings describing what was created.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError()

    def _get_agents_md_content(self) -> str:
        """Get the AGENTS.md content specific to the builder type.

        Returns:
            The content for the AGENTS.md file.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError()

    def _copy(
        self,
        source_path: Path,
        destination: Path,
        dry_run: bool,
        config: dict[str, Any] | None = None,
    ) -> str:
        """Copy a source file to destination with optional template substitution.

        Internal helper that handles file copying with support for
        template variable substitution in language-specific conventions files.

        Args:
            source_path: Source file path to copy from.
            destination: Destination file path to copy to.
            dry_run: If True, return preview string without copying.
            config: Optional config dict for template variable substitution.

        Returns:
            Action string describing the copy operation.

        Example:
            >>> action = self._copy(
            ...     Path("core-conventions-python.md"),
            ...     Path(".kilocode/rules/conventions-python.md"),
            ...     False,
            ...     {"spec": {"language": "python"}}
            ... )
            >>> print(action)
            ✓ core-conventions-python.md → .kilocode/rules/conventions-python.md
        """
        rel = str(destination).split(".kilocode/", 1)[-1]
        label = f".kilocode/{rel}"
        if dry_run:
            return f"[dry-run] {source_path.name} → {label}"
        destination.parent.mkdir(parents=True, exist_ok=True)

        # If config is provided and this is a language-specific conventions file,
        # perform template substitution
        if config and source_path.name.startswith("core-"):
            content = source_path.read_text(encoding="utf-8")
            content = self._substitute_template_variables(content, config)
            destination.write_text(content, encoding="utf-8")
        else:
            shutil.copy2(source_path, destination)

        return f"✓ {source_path.name} → {label}"

    def _substitute_template_variables(self, content: str, config: dict[str, Any]) -> str:
        """Replace {{VARIABLE}} templates with values from config.

        This method performs template variable substitution on convention
        files, replacing placeholders like {{LANGUAGE}} with actual values
        from the configuration.

        Supported variables:
            - {{LANGUAGE}}: Programming language
            - {{RUNTIME}}: Runtime version
            - {{PACKAGE_MANAGER}}: Package manager
            - {{LINTER}}: Linter tool
            - {{FORMATTER}}: Formatter tool
            - {{ABSTRACT_CLASS_STYLE}}: Abstract class style
            - {{TESTING_FRAMEWORK}}: Testing framework
            - {{TEST_RUNNER}}: Test runner
            - {{LINE_COVERAGE_%}}: Line coverage target
            - {{BRANCH_COVERAGE_%}}: Branch coverage target
            - {{FUNCTION_COVERAGE_%}}: Function coverage target
            - {{STATEMENT_COVERAGE_%}}: Statement coverage target
            - {{MUTATION_COVERAGE_%}}: Mutation coverage target
            - {{PATH_COVERAGE_%}}: Path coverage target

        Args:
            content: The template content with {{VARIABLE}} placeholders.
            config: Configuration dict containing spec values.

        Returns:
            Content with all template variables replaced.

        Example:
            >>> content = "Language: {{LANGUAGE}}"
            >>> config = {"spec": {"language": "python"}}
            >>> result = self._substitute_template_variables(content, config)
            >>> print(result)
            Language: python
        """
        defaults = config.get("spec", {}) if config else {}
        # Handle both single-language (dict) and multi-language (list) configs
        if isinstance(defaults, list):
            # Multi-language monorepo: get config from first folder
            defaults = defaults[0] if defaults else {}

        def format_value(value: Any) -> str:
            """Format a value for substitution, handling lists."""
            if isinstance(value, list):
                return ", ".join(str(v) for v in value)
            return str(value) if value is not None else ""

        # Build mapping of template variables to config values
        substitutions: dict[str, str] = {
            "{{LANGUAGE}}": format_value(defaults.get("language", "")),
            "{{RUNTIME}}": format_value(defaults.get("runtime", "")),
            "{{PACKAGE_MANAGER}}": format_value(defaults.get("package_manager", "")),
            "{{LINTER}}": format_value(defaults.get("linter", "")),
            "{{FORMATTER}}": format_value(defaults.get("formatter", "")),
            "{{ABSTRACT_CLASS_STYLE}}": format_value(defaults.get("abstract_class_style", "")),
            "{{TESTING_FRAMEWORK}}": format_value(defaults.get("test_framework", "")),
            "{{TEST_RUNNER}}": format_value(defaults.get("test_runner", "")),
        }

        # Add coverage variables
        # Coverage can be either a dict (legacy) or a string preset from the question
        coverage_value = defaults.get("coverage", {})

        # If coverage is a string preset, convert it to a dict
        if isinstance(coverage_value, str):
            COVERAGE_PRESETS = {
                "strict": {
                    "line": 90,
                    "branch": 80,
                    "function": 95,
                    "statement": 90,
                    "mutation": 85,
                    "path": 70,
                },
                "standard": {
                    "line": 80,
                    "branch": 70,
                    "function": 90,
                    "statement": 85,
                    "mutation": 80,
                    "path": 60,
                },
                "minimal": {
                    "line": 70,
                    "branch": 60,
                    "function": 80,
                    "statement": 75,
                    "mutation": 70,
                    "path": 50,
                },
            }
            coverage = COVERAGE_PRESETS.get(coverage_value, {})
        else:
            coverage = coverage_value if isinstance(coverage_value, dict) else {}

        substitutions["{{LINE_COVERAGE_%}}"] = str(coverage.get("line", 80))
        substitutions["{{BRANCH_COVERAGE_%}}"] = str(coverage.get("branch", 70))
        substitutions["{{FUNCTION_COVERAGE_%}}"] = str(coverage.get("function", 90))
        substitutions["{{STATEMENT_COVERAGE_%}}"] = str(coverage.get("statement", 85))
        substitutions["{{MUTATION_COVERAGE_%}}"] = str(coverage.get("mutation", 80))
        substitutions["{{PATH_COVERAGE_%}}"] = str(coverage.get("path", 60))

        # Perform substitutions
        for template_var, value in substitutions.items():
            content = content.replace(template_var, value)

        return content

    def _write_manifest(self, destination: Path, dry_run: bool) -> str:
        """Write the .kilocodemodes manifest file.

        Simply copies the kilo_modes.yaml source file to preserve exact formatting.

        Args:
            destination: Path where the manifest file will be written.
            dry_run: If True, return preview string without writing.

        Returns:
            Action string describing the operation.

        Example:
            >>> action = self._write_manifest(Path(".kilocodemodes"), False)
            >>> print(action)
            ✓ .kilocodemodes
        """
        # Read from the source YAML file to preserve exact formatting
        source_path = Path(__file__).parent / "kilo_modes.yaml"

        if dry_run:
            return "[dry-run] .kilocodemodes (copied from kilo_modes.yaml)"

        # Copy the file content directly
        content = source_path.read_text(encoding="utf-8")
        destination.write_text(content, encoding="utf-8")
        return "✓ .kilocodemodes"

    def _build_ignore(self, output: Path, dry_run: bool) -> list[str]:
        """Generate .kiloignore file.

        Helper method that uses KiloIgnoreBuilder to generate the
        ignore file for Kilo Code.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the ignore file.
        """
        return KiloIgnoreBuilder().build(output, dry_run)

    def _create_base_md(
        self,
        rules_dir: Path,
        language_file: str | None,
        dry_run: bool,
        config: dict[str, Any] | None = None,
    ) -> str:
        """Create _base.md by concatenating core files.

        Creates the base rules file by combining core convention files
        and optionally adding language-specific conventions.

        Args:
            rules_dir: The rules directory path.
            language_file: Optional language conventions filename.
            dry_run: If True, return preview without writing.
            config: Optional config for template substitution.

        Returns:
            Action string describing the operation.

        Example:
            >>> action = self._create_base_md(
            ...     Path(".opencode/rules"),
            ...     "core-conventions-python.md",
            ...     False,
            ...     {"spec": {"language": "python"}}
            ... )
            >>> print(action)
            ✓ .opencode/rules/_base.md
        """
        destination = rules_dir / "_base.md"
        label = ".opencode/rules/_base.md"

        if dry_run:
            return f"[dry-run] {label}"

        # Collect content from base files
        parts: list[str] = []

        for filename in self._base_files:
            source_path = registry.prompt_path(filename)
            if source_path.exists():
                content = source_path.read_text(encoding="utf-8")
                parts.append(HeaderStripper.strip(content))

        # Add language-specific conventions if selected
        if language_file:
            source_path = registry.prompt_path(language_file)
            if source_path.exists():
                content = source_path.read_text(encoding="utf-8")
                # Apply template substitution for language files
                if config:
                    content = self._substitute_template_variables(content, config)
                parts.append(HeaderStripper.strip(content))

        # Join all parts with clear separators
        full_content = "\n---\n\n".join(parts)

        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(full_content, encoding="utf-8")
        return f"✓ {label}"

    def _create_collapsed_mode_md(
        self,
        rules_dir: Path,
        mode_key: str,
        filenames: list[str],
        dry_run: bool,
        config: dict[str, Any] | None = None,
    ) -> str:
        """Create a collapsed {MODE}.md file from multiple subagent files.

        Creates a single markdown file that contains all the subagent
        content for a given mode, joined together with separators.

        Args:
            rules_dir: The rules directory path.
            mode_key: The mode identifier (e.g., 'code', 'architect').
            filenames: List of prompt filenames to include.
            dry_run: If True, return preview without writing.
            config: Optional config for template substitution.

        Returns:
            Action string describing the operation.

        Example:
            >>> action = self._create_collapsed_mode_md(
            ...     Path(".opencode/rules"),
            ...     "code",
            ...     ["code-feature.md", "code-refactor.md"],
            ...     False
            ... )
            >>> print(action)
            ✓ .opencode/rules/code.md
        """
        destination = rules_dir / f"{mode_key}.md"
        label = f".opencode/rules/{mode_key}.md"

        if dry_run:
            return f"[dry-run] {label}"

        # Collect content from all subagent files
        parts: list[str] = []

        for filename in filenames:
            source_path = registry.prompt_path(filename)
            if source_path.exists():
                content = source_path.read_text(encoding="utf-8")
                parts.append(HeaderStripper.strip(content))

        # Join all parts with clear separators
        full_content = "\n---\n\n".join(parts)

        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(full_content, encoding="utf-8")
        return f"✓ {label}"

    def _create_agents_md(self, output: Path, dry_run: bool) -> str:
        """Create AGENTS.md user guide.

        Generates the AGENTS.md file that serves as the user guide
        for the Kilo Code configuration.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            Action string describing the operation.

        Example:
            >>> action = self._create_agents_md(Path("."), False)
            >>> print(action)
            ✓ AGENTS.md
        """
        destination = output / "AGENTS.md"
        label = "AGENTS.md"

        # Get builder-specific content
        content = self._get_agents_md_content()

        if dry_run:
            return f"[dry-run] {label}"

        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(content, encoding="utf-8")
        return f"✓ {label}"



# Swift test framework question

from promptosaurus.questions.base.question import Question


class SwiftTestFrameworkQuestion(Question):
    """Question handler for Swift test framework."""

    @property
    def key(self) -> str:
        return "swift_test_framework"

    @property
    def question_text(self) -> str:
        return "What test framework do you want to use for Swift?"

    @property
    def explanation(self) -> str:
        return """Test frameworks provide structure for writing and running tests.

- XCTest is Apple's official testing framework, integrated with Xcode
- Quick is a BDD-style framework inspired by RSpec"""

    @property
    def options(self) -> list[str]:
        return ["xctest", "quick"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "xctest": "Apple official - integrated with Xcode, XCTestCase, recommended",
            "quick": "BDD style - RSpec-inspired, Nimble matchers, behavior-driven",
        }

    @property
    def default(self) -> str:
        return "xctest"



# Python coverage tool question

from promptosaurus.questions.base.question import Question


class PythonCoverageToolQuestion(Question):
    """Question for Python code coverage tool selection."""

    @property
    def key(self) -> str:
        return "coverage_tool"

    @property
    def question_text(self) -> str:
        return "What coverage tool do you want to use?"

    @property
    def explanation(self) -> str:
        return """Coverage tool measures how much of your code is tested:
- pytest-cov: Industry standard, integrates with pytest, widely used
- coverage.py: The classic coverage tool, works with any test runner"""

    @property
    def options(self) -> list[str]:
        return ["pytest-cov", "coverage.py"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "pytest-cov": "Industry standard - integrates seamlessly with pytest, great reporting (recommended)",
            "coverage.py": "Classic tool - works with any test runner, standalone coverage analysis",
        }

    @property
    def default(self) -> str:
        return "pytest-cov"



"""Builder for Cline .clinerules file and .clineignore.

This module provides the ClineBuilder class that generates the configuration
files required by Cline (formerly Claude Dev) AI assistant.

Output:
  {output}/.clinerules   ← all rules concatenated with section headers
  {output}/.clineignore ← ignore patterns

Classes:
    ClineBuilder: Generates .clinerules and .clineignore for Cline.

Example:
    >>> from pathlib import Path
    >>> from promptosaurus.builders.cline import ClineBuilder
    >>> builder = ClineBuilder()
    >>> actions = builder.build(Path("./output"))
    >>> print(actions)
    ['✓ .clinerules (150 lines)', '✓ .clineignore (25 lines)']
"""

from pathlib import Path
from typing import Any

from promptosaurus.builders.builder import Builder
from promptosaurus.builders.ignore_generator import ClineIgnoreBuilder


class ClineBuilder(Builder):
    """Builder for Cline .clinerules file.

    This builder creates two files for Cline:
    - .clinerules: All rules concatenated with section headers
    - .clineignore: Ignore patterns for Cline

    The concatenated .clinerules file combines all prompts from the registry
    into a single file with clear section headers for each prompt category.

    Attributes:
        Inherits _base_files from Builder base class.

    Example:
        >>> builder = ClineBuilder()
        >>> # Build files
        >>> actions = builder.build(Path("./my-project"))
        >>> # Check output
        >>> for action in actions:
        ...     print(action)
    """

    def build(
        self, output: Path, config: dict[str, Any] | None = None, dry_run: bool = False
    ) -> list[str]:
        """Write .clinerules and .clineignore under `output`.

        Generates the Cline configuration files by:
        1. Building a concatenated .clinerules file from all registry prompts
        2. Building a .clineignore file with ignore patterns

        Args:
            output: Directory path where the files will be created.
            config: Optional configuration dict with template variables (unused for Cline).
            dry_run: If True, preview what would be written without touching filesystem.

        Returns:
            List of action strings describing what was created.

        Example:
            >>> from pathlib import Path
            >>> builder = ClineBuilder()
            >>> # Normal run
            >>> actions = builder.build(Path("./output"))
            >>> print(actions)
            ['✓ .clinerules (150 lines)', '✓ .clineignore (25 lines)']
            >>> # Dry run
            >>> actions = builder.build(Path("./output"), dry_run=True)
            >>> print(actions)
            ['[dry-run] .clinerules (150 lines)', '[dry-run] .clineignore (25 lines)']
        """
        actions: list[str] = []

        # Build .clinerules
        destination = output / ".clinerules"
        content = self._build_concatenated("# .clinerules")

        if dry_run:
            lines = content.count("\n")
            actions.append(f"[dry-run] .clinerules ({lines} lines)")
        else:
            destination.parent.mkdir(parents=True, exist_ok=True)
            destination.write_text(content, encoding="utf-8")
            lines = content.count("\n")
            actions.append(f"✓ .clinerules ({lines} lines)")

        # Build .clineignore
        actions.extend(self._build_ignore(output, dry_run))

        return actions

    def _build_ignore(self, output: Path, dry_run: bool) -> list[str]:
        """Generate .clineignore file.

        Helper method that uses ClineIgnoreBuilder to generate the
        ignore file for Cline.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the ignore file.
        """
        return ClineIgnoreBuilder().build(output, dry_run)



# Swift version question

from promptosaurus.questions.base.question import Question


class SwiftVersionQuestion(Question):
    """Question handler for Swift version."""

    @property
    def key(self) -> str:
        return "swift_version"

    @property
    def question_text(self) -> str:
        return "What Swift version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Swift version affects available features, performance, and platform support.

- Newer versions have improved performance and language features
- Swift 5.9+ includes macros and improved C++ interoperability
- Version affects minimum iOS/macOS deployment targets"""

    @property
    def options(self) -> list[str]:
        return ["5.9", "5.8", "5.7"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "5.9": "Latest stable - macros, improved C++ interop, parameter packs, recommended",
            "5.8": "Recent stable - improved performance, better concurrency",
            "5.7": "Stable - regex literals, concurrency improvements, opaque types",
        }

    @property
    def default(self) -> str:
        return "5.9"



# TypeScript version question

from promptosaurus.questions.base.question import Question


class TypeScriptVersionQuestion(Question):
    """Question handler for TypeScript version."""

    @property
    def key(self) -> str:
        return "typescript_version"

    @property
    def question_text(self) -> str:
        return "What TypeScript version do you want to use?"

    @property
    def explanation(self) -> str:
        return """TypeScript version affects available features and type system capabilities.

- Newer versions have better inference and more features
- Older versions have better ecosystem compatibility"""

    @property
    def options(self) -> list[str]:
        return ["5.4", "5.3", "5.2", "5.1", "5.0"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "5.4": "Latest stable - best inference, const type params, recommended",
            "5.3": "Recent stable - excellent all-around",
            "5.2": "Stable - decorators,/modifiers, narrowing",
            "5.1": "Long-term support - very stable, maximum compatibility",
            "5.0": "Major release - significant changes, may need updates",
        }

    @property
    def default(self) -> str:
        return "5.4"



"""Artifact management for AI tool configuration files.

This module provides the ArtifactManager class for managing AI tool-specific
configuration artifacts. It tracks which files each AI tool creates and removes
when switching between tools.

The ARTIFACT_FILES constant defines:
    - Which files/directories each tool creates
    - Which files/directories should be removed when switching to another tool

This ensures clean transitions between different AI assistant configurations
without leaving orphaned files.

Classes:
    ArtifactManager: Manages AI tool artifact creation and removal.

Constants:
    ARTIFACT_FILES: Dictionary mapping tool names to their create/remove artifacts.

Example:
    >>> from promptosaurus.artifacts import ArtifactManager
    >>> manager = ArtifactManager(Path('.'))
    >>> actions = manager.remove_artifacts('kilo-cli')
    >>> # Removes .kilocode/, .clinerules, .cursor/, etc.
"""

import shutil
from pathlib import Path
from typing import Final

# Define which artifacts each tool creates and should remove when switching
ARTIFACT_FILES: Final[dict[str, dict[str, set[str]]]] = {
    "kilo-cli": {
        "create": {".opencode/"},
        "remove": {
            ".kilocode/",
            ".clinerules",
            ".cursor/",
            ".cursorrules",
            ".github/copilot-instructions.md",
        },
    },
    "kilo-ide": {
        "create": {".kilocode/"},
        "remove": {
            ".opencode/",
            ".clinerules",
            ".cursor/",
            ".cursorrules",
            ".github/copilot-instructions.md",
        },
    },
    "cline": {
        "create": {".clinerules"},
        "remove": {
            ".opencode/",
            ".kilocode/",
            ".cursor/",
            ".cursorrules",
            ".github/copilot-instructions.md",
        },
    },
    "cursor": {
        "create": {".cursor/", ".cursorrules"},
        "remove": {
            ".opencode/",
            ".kilocode/",
            ".clinerules",
            ".github/copilot-instructions.md",
        },
    },
    "copilot": {
        "create": {".github/copilot-instructions.md"},
        "remove": {
            ".opencode/",
            ".kilocode/",
            ".clinerules",
            ".cursor/",
            ".cursorrules",
        },
    },
}


class ArtifactManager:
    """Manage AI tool artifact creation and removal.

    This class handles cleaning up old AI tool artifacts when switching to a
    new tool, and provides information about what artifacts each tool creates.

    Attributes:
        base_path: Root path for artifact operations.

    Example:
        >>> manager = ArtifactManager(Path('/project'))
        >>> manager.remove_artifacts('kilo-cli')
        ['Removed directory: .kilocode/', 'Removed file: .clinerules']
    """

    def __init__(self, base_path: Path | None = None) -> None:
        """Initialize the artifact manager.

        Args:
            base_path: Base path for artifact operations. Defaults to current directory.
        """
        self.base_path = base_path if base_path is not None else Path(".")

    def remove_artifacts(self, tool: str) -> list[str]:
        """Remove artifacts for a specific tool.

        Removes all artifact files/directories that the specified tool does NOT create,
        effectively cleaning up artifacts from other tools.

        Args:
            tool: The AI tool name (e.g., 'kilo-cli', 'cline', 'cursor').

        Returns:
            List of action messages describing what was removed.
            Empty list if tool is not recognized.

        Raises:
            OSError: If file/directory removal fails.

        Example:
            >>> manager = ArtifactManager(Path('.'))
            >>> actions = manager.remove_artifacts('kilo-cli')
            >>> print(actions)
            ['Removed directory: .kilocode/', 'Removed file: .clinerules']
        """
        if tool not in ARTIFACT_FILES:
            return []

        to_remove = ARTIFACT_FILES[tool]["remove"]
        actions: list[str] = []

        for artifact in to_remove:
            artifact_path = self.base_path / artifact

            if artifact_path.exists():
                try:
                    if artifact_path.is_dir():
                        shutil.rmtree(artifact_path)
                        actions.append(f"Removed directory: {artifact}")
                    else:
                        artifact_path.unlink()
                        actions.append(f"Removed file: {artifact}")
                except OSError as e:
                    actions.append(f"Failed to remove {artifact}: {e}")

        return actions

    @property
    def current_tool(self) -> str | None:
        """Detect currently configured AI tool by checking which artifacts exist.

        Returns:
            The name of the currently active tool, or None if no tool detected.
            Returns the first matching tool found.

        Example:
            >>> manager = ArtifactManager(Path('.'))
            >>> tool = manager.current_tool
            >>> print(tool)  # 'kilo-cli', 'cursor', etc., or None
        """
        for tool, files in ARTIFACT_FILES.items():
            # Check if any of this tool's unique files exist
            for artifact in files["create"]:
                if (self.base_path / artifact).exists():
                    return tool

        return None

    def get_artifacts_to_create(self, tool: str) -> set[str]:
        """Get the set of artifacts that should be created for a tool.

        Args:
            tool: The AI tool name (e.g., 'kilo-cli', 'cline').

        Returns:
            Set of artifact paths to create. Empty set if tool not recognized.
        """
        if tool not in ARTIFACT_FILES:
            return set()
        return ARTIFACT_FILES[tool]["create"]

    def get_artifacts_to_remove(self, tool: str) -> set[str]:
        """Get the set of artifacts that should be removed for a tool.

        Args:
            tool: The AI tool name (e.g., 'kilo-cli', 'cline').

        Returns:
            Set of artifact paths to remove. Empty set if tool not recognized.
        """
        if tool not in ARTIFACT_FILES:
            return set()
        return ARTIFACT_FILES[tool]["remove"]



# Rust version question

from promptosaurus.questions.base.question import Question


class RustVersionQuestion(Question):
    """Question for Rust version selection."""

    @property
    def key(self) -> str:
        return "rust_version"

    @property
    def question_text(self) -> str:
        return "What Rust version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Rust version affects language features, performance, and crate compatibility.

- Newer versions have better performance and compiler improvements
- Edition 2021 is the current standard (enabled by default in 1.56+)
- Async/await has been stable since 1.39
- Const generics have improved significantly in recent versions"""

    @property
    def options(self) -> list[str]:
        return ["1.75", "1.74", "1.73", "1.72"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "1.75": "Latest stable - const trait implementations, pointer byte offsets",
            "1.74": "Very stable - lint configuration through Cargo.toml, improved async fn",
            "1.73": "Stable - cleaner panic messages, new APIs in std",
            "1.72": "Mature - stable const traits in limited form, good compatibility",
        }

    @property
    def default(self) -> str:
        return "1.75"



"""Builder for Cursor rule files and .cursorignore.

This module provides the CursorBuilder class that generates the configuration
files required by Cursor AI editor.

Output:
  {output}/.cursor/rules/core-system.mdc          ← always-on as .mdc
  {output}/.cursor/rules/core.mdc
  {output}/.cursor/rules/{mode}/{topic}.mdc        ← per-mode as .mdc
  {output}/.cursorrules                            ← legacy fallback (concatenated)
  {output}/.cursorignore                           ← ignore patterns

Classes:
    CursorBuilder: Generates Cursor rule files in multiple formats.

Example:
    >>> from pathlib import Path
    >>> from promptosaurus.builders.cursor import CursorBuilder
    >>> builder = CursorBuilder()
    >>> actions = builder.build(Path("./output"))
    >>> for action in actions[:3]:
    ...     print(action)
    ✓ core-system.mdc → .cursor/rules/core-system.mdc
    ✓ core-conventions.mdc → .cursor/rules/core-conventions.mdc
    ✓ core-session.mdc → .cursor/rules/core-session.mdc
"""

import shutil
from pathlib import Path
from typing import Any

from promptosaurus.builders.builder import Builder
from promptosaurus.builders.ignore_generator import CursorIgnoreBuilder
from promptosaurus.registry import registry


class CursorBuilder(Builder):
    """Builder for Cursor rule files.

    This builder creates multiple output formats for Cursor:
    - Individual .mdc files in .cursor/rules/ directory for always-on rules
    - Per-mode .mdc files in subdirectories
    - Legacy .cursorrules file as fallback
    - .cursorignore file with ignore patterns

    Cursor uses .mdc (Markdown Cursor) files as its rule format, which
    are processed as markdown with special Cursor directives.

    Attributes:
        Inherits _base_files from Builder base class.

    Example:
        >>> builder = CursorBuilder()
        >>> # Build all files
        >>> actions = builder.build(Path("./my-project"))
        >>> # Check results
        >>> print(f\"Generated {len(actions)} files\")
    """

    def build(
        self, output: Path, config: dict[str, Any] | None = None, dry_run: bool = False
    ) -> list[str]:
        """Write Cursor rule files under `output`.

        Generates Cursor configuration by:
        1. Creating always-on rules as .mdc files in .cursor/rules/
        2. Creating per-mode rules as .mdc files in mode subdirectories
        3. Creating legacy .cursorrules fallback file
        4. Creating .cursorignore file

        Args:
            output: Directory path where the files will be created.
            config: Optional configuration dict with template variables (unused for Cursor).
            dry_run: If True, preview what would be written without touching filesystem.

        Returns:
            List of action strings describing what was created.

        Example:
            >>> from pathlib import Path
            >>> builder = CursorBuilder()
            >>> # Normal run
            >>> actions = builder.build(Path("./output"))
            >>> # Dry run
            >>> actions = builder.build(Path("./output"), dry_run=True)
        """
        actions: list[str] = []
        rules_base = output / ".cursor" / "rules"

        # Always-on rules as .mdc
        for filename in registry.always_on:
            mdc_name = filename.replace(".md", ".mdc")
            destination = rules_base / mdc_name
            actions.append(self._copy(registry.prompt_path(filename), destination, dry_run))

        # Per-mode rules as .mdc in subdirectories
        for mode_key, files in registry.mode_files.items():
            for filename in files:
                dname = registry.dest_name(mode_key, filename, ext=".mdc")
                destination = rules_base / mode_key / dname
                actions.append(self._copy(registry.prompt_path(filename), destination, dry_run))

        # Legacy .cursorrules fallback
        legacy_dst = output / ".cursorrules"
        content = self._build_concatenated("# .cursorrules")
        if dry_run:
            actions.append(f"[dry-run] .cursorrules ({content.count(chr(10))} lines, legacy)")
        else:
            legacy_dst.parent.mkdir(parents=True, exist_ok=True)
            legacy_dst.write_text(content, encoding="utf-8")
            actions.append(f"✓ .cursorrules ({content.count(chr(10))} lines, legacy fallback)")

        # Build .cursorignore
        actions.extend(self._build_ignore(output, dry_run))

        return actions

    def _build_ignore(self, output: Path, dry_run: bool) -> list[str]:
        """Generate .cursorignore file.

        Helper method that uses CursorIgnoreBuilder to generate the
        ignore file for Cursor.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the ignore file.
        """
        return CursorIgnoreBuilder().build(output, dry_run)

    def _copy(self, source_path: Path, destination: Path, dry_run: bool) -> str:
        """Copy a source file to destination.

        Internal helper that handles the file copying operation with
        proper path formatting for display.

        Args:
            source_path: Source file path to copy from.
            destination: Destination file path to copy to.
            dry_run: If True, return preview string without copying.

        Returns:
            Action string describing the copy operation.

        Example:
            >>> action = self._copy(Path("core-system.md"), Path(".cursor/rules/core-system.mdc"), False)
            >>> print(action)
            ✓ core-system.mdc → .cursor/rules/core-system.mdc
        """
        rel = str(destination).split(".cursor/", 1)[-1]
        label = f".cursor/{rel}"
        if dry_run:
            return f"[dry-run] {source_path.name} → {label}"
        destination.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy2(source_path, destination)
        return f"✓ {source_path.name} → {label}"



"""Interface and implementations for generating ignore files.

This module provides builder classes for generating various ignore files
used by different AI assistant tools. Each builder creates a tool-specific
ignore file that prevents the AI from processing certain files.

Uses interface pattern (NotImplementedError) per core-conventions -
no ABC, just raise NotImplementedError for required methods.

Classes:
    IgnoreFileBuilder: Abstract base interface for ignore file builders.
    KiloIgnoreBuilder: Generates .kiloignore content for Kilo Code.
    ClineIgnoreBuilder: Generates .clineignore content for Cline.
    CursorIgnoreBuilder: Generates .cursorignore content for Cursor.
    CopilotIgnoreBuilder: Generates .copilotignore content for GitHub Copilot.
    GitIgnoreBuilder: Generates .gitignore content.

Example:
    >>> from promptosaurus.builders.ignore_generator import KiloIgnoreBuilder
    >>> builder = KiloIgnoreBuilder()
    >>> print(builder.filename)
    .kiloignore
"""

from pathlib import Path

from promptosaurus.registry import registry


class IgnoreFileBuilder:
    """Interface for ignore file builders.

    This abstract class defines the interface for generating tool-specific
    ignore files. Subclasses must implement the filename and content properties.

    Uses interface pattern (NotImplementedError) per conventions -
    no ABC, just raise NotImplementedError for required methods.

    Attributes:
        filename: Property that returns the ignore filename.
        content: Property that returns the ignore file content.

    Methods:
        build: Generate and write the ignore file.

    Example:
        >>> class MyIgnoreBuilder(IgnoreFileBuilder):
        ...     @property
        ...     def filename(self) -> str:
        ...         return ".myignore"
        ...     @property
        ...     def content(self) -> str:
        ...         return "*.log\\n"
    """

    @property
    def filename(self) -> str:
        """Filename for the ignore file (e.g., '.kiloignore').

        Returns:
            The filename string including the leading dot.

        Raises:
            NotImplementedError: If subclass doesn't implement this property.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement filename")

    @property
    def content(self) -> str:
        """Content for the ignore file.

        Returns:
            The complete ignore file content as a string.

        Raises:
            NotImplementedError: If subclass doesn't implement this property.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement content")

    def build(self, output: Path, dry_run: bool = False) -> list[str]:
        """Build the ignore file.

        Generates the ignore file in the specified output directory.

        Args:
            output: Output directory path where the ignore file will be created.
            dry_run: If True, return preview string without writing to filesystem.

        Returns:
            List containing a single action string describing the operation.

        Raises:
            FileExistsError: If the file already exists and cannot be overwritten.

        Example:
            >>> from pathlib import Path
            >>> from promptosaurus.builders.ignore_generator import KiloIgnoreBuilder
            >>> builder = KiloIgnoreBuilder()
            >>> actions = builder.build(Path("./output"))
            >>> print(actions)
            ['✓ .kiloignore (42 lines)']
        """
        destination = output / self.filename
        if dry_run:
            lines = self.content.count("\n")
            return [f"[dry-run] {self.filename} ({lines} lines)"]
        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(self.content, encoding="utf-8")
        lines = self.content.count("\n")
        return [f"✓ {self.filename} ({lines} lines)"]


class KiloIgnoreBuilder(IgnoreFileBuilder):
    """Generates .kiloignore content for Kilo Code.

    This builder creates the ignore file that tells Kilo Code which files
    to exclude from processing. Content is generated from the registry.

    Example:
        >>> builder = KiloIgnoreBuilder()
        >>> print(builder.filename)
        .kiloignore
    """

    @property
    def filename(self) -> str:
        return ".kiloignore"

    @property
    def content(self) -> str:
        return registry.generate_kiloignore()


class ClineIgnoreBuilder(IgnoreFileBuilder):
    """Generates .clineignore content for Cline.

    This builder creates the ignore file that tells Cline which files
    to exclude from processing. Content is generated from the registry.

    Example:
        >>> builder = ClineIgnoreBuilder()
        >>> print(builder.filename)
        .clineignore
    """

    @property
    def filename(self) -> str:
        return ".clineignore"

    @property
    def content(self) -> str:
        return registry.generate_clineignore()


class CursorIgnoreBuilder(IgnoreFileBuilder):
    """Generates .cursorignore content for Cursor.

    This builder creates the ignore file that tells Cursor which files
    to exclude from processing. Content is generated from the registry.

    Example:
        >>> builder = CursorIgnoreBuilder()
        >>> print(builder.filename)
        .cursorignore
    """

    @property
    def filename(self) -> str:
        return ".cursorignore"

    @property
    def content(self) -> str:
        return registry.generate_cursorignore()


class CopilotIgnoreBuilder(IgnoreFileBuilder):
    """Generates .copilotignore content for GitHub Copilot.

    This builder creates the ignore file that tells GitHub Copilot which files
    to exclude from processing. Content is generated from the registry.

    Example:
        >>> builder = CopilotIgnoreBuilder()
        >>> print(builder.filename)
        .copilotignore
    """

    @property
    def filename(self) -> str:
        return ".copilotignore"

    @property
    def content(self) -> str:
        return registry.generate_copilotignore()


class GitIgnoreBuilder(IgnoreFileBuilder):
    """Generates .gitignore content.

    This builder creates a standard .gitignore file with common ignore patterns
    for Python projects. Content is generated from the registry.

    Example:
        >>> builder = GitIgnoreBuilder()
        >>> print(builder.filename)
        .gitignore
    """

    @property
    def filename(self) -> str:
        return ".gitignore"

    @property
    def content(self) -> str:
        return registry.generate_gitignore()



"""Column layout renderer."""

from promptosaurus.ui.domain.context import PipelineContext
from promptosaurus.ui.render.renderer import Renderer


class ColumnLayoutRenderer(Renderer):
    """Renders options in columns."""

    def __init__(self, items_per_column: int = 8, column_width: int = 20):
        self.items_per_column = items_per_column
        self.column_width = column_width

    def render(self, context: PipelineContext) -> str:
        """Render options in column layout."""
        options = context.display_options
        lines = []

        num_items = len(options)
        num_columns = (num_items + self.items_per_column - 1) // self.items_per_column

        for row in range(self.items_per_column):
            line_parts = []
            for col in range(num_columns):
                idx = col * self.items_per_column + row
                if idx < num_items:
                    num_str = f"{idx + 1}."
                    content = f"{num_str:>3} {options[idx]}"
                    part = content.ljust(self.column_width)
                    line_parts.append(part)
            if line_parts:
                lines.append("".join(line_parts))

        return "\n".join(lines)



# Go version question

from promptosaurus.questions.base.question import Question


class GoVersionQuestion(Question):
    """Question handler for Go version selection."""

    @property
    def key(self) -> str:
        return "go_version"

    @property
    def question_text(self) -> str:
        return "What Go version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Go version affects module compatibility, performance, and language features.

- Newer versions have better performance and runtime improvements
- Go modules (go.mod) became the standard in 1.13+
- Generics support was added in Go 1.18
- Some libraries require minimum Go versions"""

    @property
    def options(self) -> list[str]:
        return ["1.22", "1.21", "1.20", "1.19"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "1.22": "Latest stable - improved HTTP routing, better loop variable semantics",
            "1.21": "Very stable - built-in min/max functions, improved slog package",
            "1.20": "Stable - context with cancel cause, unsafe.SliceData, strings.CutPrefix",
            "1.19": "Mature - generics available, minimal changes, maximum compatibility",
        }

    @property
    def default(self) -> str:
        return "1.22"



# Shell linter question

from promptosaurus.questions.base.question import Question


class ShellLinterQuestion(Question):
    """Question handler for shell linter."""

    @property
    def key(self) -> str:
        return "shell_linter"

    @property
    def question_text(self) -> str:
        return "What linter do you want to use for shell scripts?"

    @property
    def explanation(self) -> str:
        return """Shell linters catch bugs and enforce best practices.

- ShellCheck is the industry standard for shell script analysis
- It catches common mistakes and security issues
- Highly recommended for all shell scripting"""

    @property
    def options(self) -> list[str]:
        return ["shellcheck"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "shellcheck": "Industry standard - comprehensive analysis, catches bugs",
        }

    @property
    def default(self) -> str:
        return "shellcheck"



"""Input provider interface for the UI domain.

This module defines the abstract interface for input providers,
which handle platform-specific keyboard input.

Classes:
    InputProvider: Abstract base class for input providers.

Example:
    >>> from promptosaurus.ui.domain.input_provider import InputProvider
    >>>
    >>> # Implement a custom provider
    >>> class MyInputProvider(InputProvider):
    ...     @property
    ...     def events(self):
    ...         # Yield events
    ...         pass
    ...     def supports_raw(self):
    ...         return False
"""

from collections.abc import Iterator

from promptosaurus.ui.domain.events import InputEvent


class InputProvider:
    """Abstract base class for input providers.

    Input providers handle platform-specific keyboard input and convert
    raw key presses into InputEvent objects. This abstraction allows
    the UI to work on different platforms (Windows, Unix) with
    different terminal capabilities.

    Attributes:
        events: Property that yields InputEvent objects from user input.

    Methods:
        supports_raw: Check if raw input mode is supported.

    Example:
        >>> # Concrete implementations available:
        >>> from promptosaurus.ui.input.windows import WindowsInputProvider
        >>> from promptosaurus.ui.input.unix import UnixInputProvider
        >>> from promptosaurus.ui.input.fallback import FallbackInputProvider
    """

    @property
    def events(self) -> Iterator[InputEvent]:
        """Yield input events.

        This property should be implemented by subclasses to yield
        InputEvent objects as the user presses keys.

        Yields:
            InputEvent objects representing user keyboard input.

        Raises:
            NotImplementedError: If subclass doesn't implement this property.
        """
        raise NotImplementedError("Subclasses must implement the events property")

    def supports_raw(self) -> bool:
        """Whether raw input is supported.

        Returns:
            True if raw terminal input is supported, False otherwise.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError("Subclasses must implement supports_raw()")



"""Tests for prompt init questions."""

import pytest

from promptosaurus.questions.base.constants import RepositoryTypes
from promptosaurus.questions.base.folder_mapping_question import FolderMappingQuestion
from promptosaurus.questions.base.repository_type_question import RepositoryTypeQuestion
from promptosaurus.questions.handlers.handle_single_language_questions import (
    HandleSingleLanguageQuestions,
)
from promptosaurus.questions.handlers.language_question_handler import LanguageQuestionHandler
from promptosaurus.questions.language import (
    LANGUAGE_KEYS,
    get_language_questions,
)
from promptosaurus.questions.python.python_formatter_question import PythonFormatterQuestion
from promptosaurus.questions.python.python_linter_question import PythonLinterQuestion
from promptosaurus.questions.python.python_package_manager_question import (
    PythonPackageManagerQuestion,
)
from promptosaurus.questions.python.python_runtime_question import PythonRuntimeQuestion
from promptosaurus.questions.python.python_test_framework_question import (
    PythonTestFrameworkQuestion,
)
from promptosaurus.questions.python.python_test_runner_question import PythonTestRunnerQuestion
from promptosaurus.questions.typescript.typescript_package_manager_question import (
    TypeScriptPackageManagerQuestion,
)
from promptosaurus.questions.typescript.typescript_version_question import (
    TypeScriptVersionQuestion,
)


class TestRepositoryTypeQuestion:
    """Tests for RepositoryTypeQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = RepositoryTypeQuestion()

        assert q.key == "repository_type"
        assert q.question_text
        assert q.explanation
        assert q.options
        assert q.default

    def test_options_are_valid(self):
        """Options should include all valid repo types."""
        q = RepositoryTypeQuestion()

        assert RepositoryTypes.SINGLE in q.options
        assert RepositoryTypes.MULTI_MONOREPO in q.options
        assert RepositoryTypes.MIXED in q.options

    def test_default_is_single_language(self):
        """Default should be single-language."""
        q = RepositoryTypeQuestion()

        assert q.default == RepositoryTypes.SINGLE

    def test_option_explanations_exist(self):
        """Each option should have an explanation."""
        q = RepositoryTypeQuestion()

        for opt in q.options:
            assert opt in q.option_explanations
            assert q.option_explanations[opt]

    def test_explain_option_returns_explanation(self):
        """explain_option should return the explanation for an option."""
        q = RepositoryTypeQuestion()

        explanation = q.explain_option(RepositoryTypes.SINGLE)
        assert explanation == q.option_explanations[RepositoryTypes.SINGLE]

    def test_explain_option_unknown_returns_empty(self):
        """explain_option for unknown option should return empty string."""
        q = RepositoryTypeQuestion()

        assert q.explain_option("unknown") == ""


class TestFolderMappingQuestion:
    """Tests for FolderMappingQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = FolderMappingQuestion()

        assert q.key == "folder_mapping"
        assert q.question_text
        assert q.explanation

    def test_options_is_empty_list(self):
        """Options should be empty for dynamic folder mapping."""
        q = FolderMappingQuestion()

        assert q.options == []

    def test_default_is_empty_string(self):
        """Default should be empty string for folder mapping."""
        q = FolderMappingQuestion()

        assert q.default == ""


class TestPythonRuntimeQuestion:
    """Tests for PythonRuntimeQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonRuntimeQuestion()

        assert q.key == "python_runtime"
        assert q.question_text
        assert q.explanation
        assert q.options
        assert q.default

    def test_options_include_common_runtimes(self):
        """Options should include common Python runtimes."""
        q = PythonRuntimeQuestion()

        assert "3.12" in q.options
        assert "3.11" in q.options
        assert "pypy" in q.options

    def test_default_is_latest_stable(self):
        """Default should be latest stable version."""
        q = PythonRuntimeQuestion()

        assert q.default == "3.12"

    def test_option_explanations_for_all_options(self):
        """Each option should have an explanation."""
        q = PythonRuntimeQuestion()

        for opt in q.options:
            assert opt in q.option_explanations


class TestPythonPackageManagerQuestion:
    """Tests for PythonPackageManagerQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonPackageManagerQuestion()

        assert q.key == "python_package_manager"
        assert q.options

    def test_options_include_common_managers(self):
        """Options should include common Python package managers."""
        q = PythonPackageManagerQuestion()

        assert "poetry" in q.options
        assert "pip" in q.options
        assert "uv" in q.options

    def test_default_is_uv(self):
        """Default should be uv."""
        q = PythonPackageManagerQuestion()

        assert q.default == "uv"


class TestPythonTestFrameworkQuestion:
    """Tests for PythonTestFrameworkQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonTestFrameworkQuestion()

        assert q.key == "python_test_framework"
        assert q.options

    def test_options_include_common_frameworks(self):
        """Options should include common Python test frameworks."""
        q = PythonTestFrameworkQuestion()

        assert "pytest" in q.options
        assert "unittest" in q.options

    def test_default_is_hybrid(self):
        """Default should be hybrid."""
        q = PythonTestFrameworkQuestion()

        assert q.default == "hybrid"


class TestTypeScriptVersionQuestion:
    """Tests for TypeScriptVersionQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = TypeScriptVersionQuestion()

        assert q.key == "typescript_version"
        assert q.options

    def test_options_include_recent_versions(self):
        """Options should include recent TypeScript versions."""
        q = TypeScriptVersionQuestion()

        assert "5.4" in q.options
        assert "5.3" in q.options
        assert "5.0" in q.options

    def test_default_is_latest(self):
        """Default should be latest stable version."""
        q = TypeScriptVersionQuestion()

        assert q.default == "5.4"


class TestTypeScriptPackageManagerQuestion:
    """Tests for TypeScriptPackageManagerQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = TypeScriptPackageManagerQuestion()

        assert q.key == "typescript_package_manager"
        assert q.options

    def test_options_include_common_managers(self):
        """Options should include common JS package managers."""
        q = TypeScriptPackageManagerQuestion()

        assert "npm" in q.options
        assert "pnpm" in q.options
        assert "yarn" in q.options


class TestGetLanguageQuestions:
    """Tests for get_language_questions function."""

    def test_get_python_questions(self):
        """Should return questions for Python."""
        questions = get_language_questions("python")

        assert len(questions) > 0
        keys = [q.key for q in questions]
        assert "python_runtime" in keys
        assert "python_package_manager" in keys

    def test_get_typescript_questions(self):
        """Should return questions for TypeScript."""
        questions = get_language_questions("typescript")

        assert len(questions) > 0
        keys = [q.key for q in questions]
        assert "typescript_version" in keys

    def test_get_javascript_questions(self):
        """Should return questions for JavaScript (same as TypeScript)."""
        questions = get_language_questions("javascript")

        assert len(questions) > 0

    def test_get_unknown_language_returns_empty(self):
        """Unknown language should raise ValueError."""
        with pytest.raises(ValueError):
            get_language_questions("unknown_language")

    def test_language_keys_includes_common_languages(self):
        """LANGUAGE_KEYS should include common languages."""
        assert "python" in LANGUAGE_KEYS
        assert "typescript" in LANGUAGE_KEYS
        assert "javascript" in LANGUAGE_KEYS
        assert "go" in LANGUAGE_KEYS
        assert "rust" in LANGUAGE_KEYS


class TestPythonLinterQuestion:
    """Tests for PythonLinterQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonLinterQuestion()

        assert q.key == "python_linter"
        assert q.question_text
        assert q.explanation
        assert q.options

    def test_options_include_common_linters(self):
        """Options should include common Python linters."""
        q = PythonLinterQuestion()

        assert "ruff" in q.options
        assert "flake8" in q.options
        assert "pylint" in q.options
        assert "pyright" in q.options

    def test_default_is_ruff(self):
        """Default should be ruff."""
        q = PythonLinterQuestion()

        assert q.default == "ruff"

    def test_allow_multiple_is_true(self):
        """Should allow multiple selections."""
        q = PythonLinterQuestion()

        assert q.allow_multiple is True

    def test_option_explanations_for_all_options(self):
        """Each option should have an explanation."""
        q = PythonLinterQuestion()

        for opt in q.options:
            assert opt in q.option_explanations


class TestPythonFormatterQuestion:
    """Tests for PythonFormatterQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonFormatterQuestion()

        assert q.key == "python_formatter"
        assert q.question_text
        assert q.explanation
        assert q.options

    def test_options_include_common_formatters(self):
        """Options should include common Python formatters."""
        q = PythonFormatterQuestion()

        assert "ruff" in q.options
        assert "black" in q.options
        assert "yapf" in q.options

    def test_default_is_ruff(self):
        """Default should be ruff."""
        q = PythonFormatterQuestion()

        assert q.default == "ruff"

    def test_allow_multiple_is_true(self):
        """Should allow multiple selections."""
        q = PythonFormatterQuestion()

        assert q.allow_multiple is True

    def test_option_explanations_for_all_options(self):
        """Each option should have an explanation."""
        q = PythonFormatterQuestion()

        for opt in q.options:
            assert opt in q.option_explanations


class TestPythonTestRunnerQuestion:
    """Tests for PythonTestRunnerQuestion."""

    def test_question_has_required_properties(self):
        """Question should have all required properties."""
        q = PythonTestRunnerQuestion()

        assert q.key == "python_test_runner"
        assert q.question_text
        assert q.explanation
        assert q.options

    def test_options_include_common_runners(self):
        """Options should include common test runners."""
        q = PythonTestRunnerQuestion()

        assert "pytest" in q.options
        assert "nose2" in q.options
        assert "unittest" in q.options

    def test_default_is_pytest(self):
        """Default should be pytest."""
        q = PythonTestRunnerQuestion()

        assert q.default == "pytest"

    def test_option_explanations_for_all_options(self):
        """Each option should have an explanation."""
        q = PythonTestRunnerQuestion()

        for opt in q.options:
            assert opt in q.option_explanations


class TestLanguageQuestionHandler:
    """Tests for LanguageQuestionHandler base class."""

    def test_handle_method_raises_not_implemented(self):
        """Base handle() method should raise NotImplementedError."""
        handler = LanguageQuestionHandler()

        with pytest.raises(NotImplementedError):
            handler.handle("single-language")


class TestHandleSingleLanguageQuestions:
    """Tests for HandleSingleLanguageQuestions handler."""

    def test_handler_initializes_with_selector(self):
        """Handler should initialize with a selector function."""

        def mock_selector(**kwargs):  # noqa: ARG001
            return "python"

        handler = HandleSingleLanguageQuestions(mock_selector)

        assert handler.select_option == mock_selector

    def test_handle_returns_config_dict(self, monkeypatch):
        """Handle should return a configuration dictionary."""

        def mock_selector(**kwargs):  # noqa: ARG001
            return "python"

        handler = HandleSingleLanguageQuestions(mock_selector)

        # Mock click.echo to avoid output during test
        monkeypatch.setattr("click.echo", lambda x: None)

        config = handler.handle("single-language")

        assert isinstance(config, dict)
        assert "spec" in config

    def test_python_questions_in_config(self, monkeypatch):
        """Python questions should populate config defaults."""

        def mock_selector(**kwargs):  # noqa: ARG001
            return "pytest" if "pytest" in kwargs.get("options", []) else "python"

        handler = HandleSingleLanguageQuestions(mock_selector)

        # Mock click.echo to avoid output during test
        monkeypatch.setattr("click.echo", lambda x: None)

        config = handler.handle("single-language")

        assert "spec" in config
        # Config should have some spec populated
        assert isinstance(config["spec"], dict)



"""Quit command implementation."""

from promptosaurus.ui.commands.command import Command
from promptosaurus.ui.domain.context import PipelineContext
from promptosaurus.ui.exceptions import UserCancelledError


class QuitCommand(Command):
    """Command to quit/exit the CLI."""

    def execute(self, context: PipelineContext) -> None:
        """Execute quit command - raises UserCancelledError to exit CLI."""
        raise UserCancelledError("User cancelled the operation")



"""Navigate command implementation."""

from promptosaurus.ui.commands.command import Command
from promptosaurus.ui.commands.result import CommandResult
from promptosaurus.ui.domain.context import PipelineContext


class NavigateCommand(Command):
    """Command to navigate up/down."""

    def __init__(self, direction: int):
        """Initialize with direction (-1 for up, +1 for down)."""
        self.direction = direction

    def execute(self, context: PipelineContext) -> CommandResult:
        """Execute navigate command."""
        new_state = context.state.navigate(self.direction)
        return CommandResult(
            continue_pipeline=True,
            new_state=new_state,
        )



"""Configuration loader for Kilo builder settings.

This module provides the KiloConfig class that loads and manages configuration
for the Kilo builder, including custom modes and language file mappings.

The configuration is loaded from YAML files:
- kilo_modes.yaml: Custom mode definitions
- kilo_language_file_map.yaml: Language to conventions file mappings

Classes:
    KiloConfig: Configuration loader for Kilo builder settings.

Example:
    >>> from promptosaurus.builders.config import KiloConfig
    >>> config = KiloConfig()
    >>> modes = config.kilo_modes
    >>> print(len(modes) > 0)
    True
    >>> lang_map = config.language_file_map
    >>> print("python" in lang_map)
    True
"""

from pathlib import Path
from typing import Any

import yaml


class KiloConfig:
    """Configuration loader for Kilo builder settings.

    This class loads configuration from YAML files and provides
    typed access to settings. It uses lazy loading to defer
    file I/O until the properties are first accessed.

    Attributes:
        kilo_modes: Dictionary of mode slug to mode configuration.
        language_file_map: Dictionary of language name to conventions file.
        base_files: Frozenset of core file paths needed for .kilocode/rules/.

    Methods:
        _load_modes: Load custom modes from YAML file.
        _load_language_map: Load language file map from YAML file.

    Args:
        modes_path: Optional custom path to kilo_modes.yaml
        language_map_path: Optional custom path to kilo_language_file_map.yaml

    Example:
        >>> # Use default paths
        >>> config = KiloConfig()
        >>> modes = config.kilo_modes

        >>> # Use custom paths
        >>> config = KiloConfig(
        ...     modes_path=Path("./custom-modes.yaml"),
        ...     language_map_path=Path("./custom-lang-map.yaml")
        ... )
    """

    def __init__(
        self,
        modes_path: Path | None = None,
        language_map_path: Path | None = None,
    ) -> None:
        """Initialize config with optional custom paths.

        Args:
            modes_path: Optional custom path to kilo_modes.yaml.
            language_map_path: Optional custom path to kilo_language_file_map.yaml.
        """
        self._modes_path = modes_path or self._default_modes_path()
        self._language_map_path = language_map_path or self._default_language_map_path()
        self._kilo_modes: dict[str, Any] | None = None
        self._language_file_map: dict[str, str] | None = None
        # Internal class variable for core files (not a constant)
        self._base_files = frozenset(
            [
                "agents/core/core-system.md",
                "agents/core/core-conventions.md",
                "agents/core/core-session.md",
            ]
        )

    @staticmethod
    def _default_modes_path() -> Path:
        """Get the default path to kilo_modes.yaml.

        Returns:
            Path to the default kilo_modes.yaml file in the kilo directory.

        Example:
            >>> path = KiloConfig._default_modes_path()
            >>> print(path.name)
            kilo_modes.yaml
        """
        return Path(__file__).parent / "kilo" / "kilo_modes.yaml"

    @staticmethod
    def _default_language_map_path() -> Path:
        """Get the default path to kilo_language_file_map.yaml.

        Returns:
            Path to the default kilo_language_file_map.yaml file in the kilo directory.

        Example:
            >>> path = KiloConfig._default_language_map_path()
            >>> print(path.name)
            kilo_language_file_map.yaml
        """
        return Path(__file__).parent / "kilo" / "kilo_language_file_map.yaml"

    @property
    def kilo_modes(self) -> dict[str, Any]:
        """Lazy-load and return kilo modes from YAML.

        Loads the custom mode definitions from the modes YAML file.
        The file is only read once on first access, then cached.

        Returns:
            Dictionary of mode slug to mode configuration.

        Raises:
            FileNotFoundError: If the modes YAML file doesn't exist.
            yaml.YAMLError: If the YAML file is malformed.

        Example:
            >>> config = KiloConfig()
            >>> modes = config.kilo_modes
            >>> print("code" in modes)
            True
        """
        if self._kilo_modes is None:
            self._kilo_modes = self._load_modes()
        return self._kilo_modes

    @property
    def language_file_map(self) -> dict[str, str]:
        """Lazy-load and return language file map from YAML.

        Loads the language to conventions file mappings from the YAML file.
        The file is only read once on first access, then cached.

        Returns:
            Dictionary mapping language name to conventions file path.

        Raises:
            FileNotFoundError: If the language map YAML file doesn't exist.
            yaml.YAMLError: If the YAML file is malformed.

        Example:
            >>> config = KiloConfig()
            >>> lang_map = config.language_file_map
            >>> print(lang_map.get("python"))
            core-conventions-python.md
        """
        if self._language_file_map is None:
            self._language_file_map = self._load_language_map()
        return self._language_file_map

    @property
    def base_files(self) -> frozenset[str]:
        """Return set of core base files needed for .kilocode/rules/.

        These are the always-on prompt files that are included in
        every build regardless of mode.

        Returns:
            Frozenset of core file paths.

        Example:
            >>> config = KiloConfig()
            >>> base = config.base_files
            >>> print("agents/core/core-system.md" in base)
            True
        """
        return self._base_files

    def _load_modes(self) -> dict[str, Any]:
        """Load custom modes from YAML file.

        Reads and parses the kilo_modes.yaml file, extracting the
        customModes array into a dictionary keyed by mode slug.

        Returns:
            Dictionary mapping mode slug to mode configuration.

        Raises:
            FileNotFoundError: If the modes YAML file doesn't exist.
            yaml.YAMLError: If the YAML file is malformed.

        Example:
            >>> config = KiloConfig()
            >>> modes = config._load_modes()
            >>> code_mode = modes.get("code")
            >>> print(code_mode is not None)
            True
        """
        with self._modes_path.open(encoding="utf-8") as f:
            data = yaml.safe_load(f)
        modes_list = data.get("customModes", [])
        return {mode["slug"]: mode for mode in modes_list if isinstance(mode, dict)}

    def _load_language_map(self) -> dict[str, str]:
        """Load language file map from YAML file.

        Reads and parses the kilo_language_file_map.yaml file,
        extracting the language_file_map dictionary.

        Returns:
            Dictionary mapping language name to conventions file.

        Raises:
            FileNotFoundError: If the language map YAML file doesn't exist.
            yaml.YAMLError: If the YAML file is malformed.

        Example:
            >>> config = KiloConfig()
            >>> lang_map = config._load_language_map()
            >>> print(lang_map.get("python", ""))
            core-conventions-python.md
        """
        with self._language_map_path.open(encoding="utf-8") as f:
            data = yaml.safe_load(f)
        return data.get("language_file_map", {})



# Kotlin build tool question

from promptosaurus.questions.base.question import Question


class KotlinBuildToolQuestion(Question):
    """Question handler for Kotlin build tool."""

    @property
    def key(self) -> str:
        return "kotlin_build_tool"

    @property
    def question_text(self) -> str:
        return "What build tool do you want to use for Kotlin?"

    @property
    def explanation(self) -> str:
        return """Build tools manage dependencies, compilation, and project lifecycle.

- Gradle is the modern standard with excellent Kotlin DSL support
- Maven is mature, widely used in enterprise environments"""

    @property
    def options(self) -> list[str]:
        return ["gradle", "maven"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "gradle": "Modern standard - excellent Kotlin DSL, flexible, fast incremental builds",
            "maven": "Mature enterprise - declarative, extensive plugin ecosystem",
        }

    @property
    def default(self) -> str:
        return "gradle"



# Python package manager question

from promptosaurus.questions.base.question import Question


class PythonPackageManagerQuestion(Question):
    """Question for Python package manager."""

    @property
    def key(self) -> str:
        return "python_package_manager"

    @property
    def question_text(self) -> str:
        return "What package manager do you want to use for Python?"

    @property
    def explanation(self) -> str:
        return """Package manager affects:
- Dependency resolution and lock file management
- Virtual environment handling
- Build system integration
- Publishing to PyPI"""

    @property
    def options(self) -> list[str]:
        return ["uv", "poetry", "pip", "conda", "pdm"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "uv": "Extremely fast - Rust-based, recommended for new projects (replaces pip)",
            "poetry": "Modern, all-in-one - dependency + env + build, widely used",
            "pip": "Standard - simple, widely compatible, no lock file by default",
            "conda": "Data science focus - manages non-Python dependencies too",
            "pdm": "Modern PEP 582 - no virtualenv, good pyproject.toml integration",
        }

    @property
    def default(self) -> str:
        return "uv"



# Python coverage targets question

from promptosaurus.questions.base.question import Question


class PythonCoverageTargetsQuestion(Question):
    """Question for Python code coverage targets."""

    _coverage_presets = {
        "strict": '{"line": 90, "branch": 80, "function": 95, "statement": 90, "mutation": 85, "path": 70}',
        "standard": '{"line": 80, "branch": 70, "function": 90, "statement": 85, "mutation": 80, "path": 60}',
        "minimal": '{"line": 70, "branch": 60, "function": 80, "statement": 75, "mutation": 70, "path": 50}',
    }

    @property
    def key(self) -> str:
        return "coverage"

    @property
    def question_text(self) -> str:
        return "What code coverage targets do you want to enforce?"

    @property
    def explanation(self) -> str:
        return """Coverage targets define the minimum acceptable test coverage for your project.
Higher coverage ensures better code quality but requires more test effort.
These targets are used by pytest-cov and enforced in CI/CD pipelines."""

    @property
    def options(self) -> list[str]:
        return list(self._coverage_presets.keys())

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "strict": "Line: 90%, Branch: 80%, Function: 95%, Statement: 90%, Mutation: 85%, Path: 70% - For production libraries",
            "standard": "Line: 80%, Branch: 70%, Function: 90%, Statement: 85%, Mutation: 80%, Path: 60% - Balanced approach (recommended)",
            "minimal": "Line: 70%, Branch: 60%, Function: 80%, Statement: 75%, Mutation: 70%, Path: 50% - For prototypes or legacy code",
        }

    @property
    def default(self) -> str:
        return "standard"

    def explain_option(self, option: str) -> str:
        """Get explanation for a specific option."""
        return self.option_explanations.get(option, "")

    def get_value(self, option: str) -> dict[str, int] | None:
        """
        Get the coverage targets for a given option.
        Returns a dict with coverage targets.
        """
        if option in self._coverage_presets:
            import json

            return json.loads(self._coverage_presets[option])
        return None



"""Unit tests for promptosaurus.registry."""

import unittest
from pathlib import Path


class TestRegistry(unittest.TestCase):
    """Tests for the registry module."""

    def test_prompts_dir_is_path(self):
        """prompts_dir should be a Path object."""
        from promptosaurus.registry import registry

        assert isinstance(registry.prompts_dir, Path)

    def test_prompts_dir_exists(self):
        """prompts_dir should exist."""
        from promptosaurus.registry import registry

        assert registry.prompts_dir.exists()

    def test_prompts_dir_is_directory(self):
        """prompts_dir should be a directory."""
        from promptosaurus.registry import registry

        assert registry.prompts_dir.is_dir()

    def test_always_on_is_list(self):
        """always_on should be a list."""
        from promptosaurus.registry import registry

        assert isinstance(registry.always_on, list)

    def test_always_on_contains_strings(self):
        """always_on should contain string filenames."""
        from promptosaurus.registry import registry

        assert all(isinstance(f, str) for f in registry.always_on)

    def test_modes_is_dict(self):
        """modes should be a dictionary."""
        from promptosaurus.registry import registry

        assert isinstance(registry.modes, dict)

    def test_modes_not_empty(self):
        """modes should not be empty."""
        from promptosaurus.registry import registry

        assert len(registry.modes) > 0

    def test_mode_files_is_dict(self):
        """mode_files should be a dictionary."""
        from promptosaurus.registry import registry

        assert isinstance(registry.mode_files, dict)

    def test_mode_files_not_empty(self):
        """mode_files should not be empty."""
        from promptosaurus.registry import registry

        assert len(registry.mode_files) > 0

    def test_prompt_path_returns_path(self):
        """prompt_path() should return a Path."""
        from promptosaurus.registry import registry

        result = registry.prompt_path("core-system.md")
        assert isinstance(result, Path)

    def test_prompt_path_includes_filename(self):
        """prompt_path() should include the filename."""
        from promptosaurus.registry import registry

        result = registry.prompt_path("core-system.md")
        assert result.name == "core-system.md"

    def test_dest_name_strips_prefix(self):
        """dest_name() should strip the mode prefix."""
        from promptosaurus.registry import registry

        result = registry.dest_name("architect", "architect-scaffold.md")
        assert result == "scaffold.md"

    def test_dest_name_with_extension(self):
        """dest_name() should handle custom extensions."""
        from promptosaurus.registry import registry

        result = registry.dest_name("architect", "architect-scaffold.md", ext=".mdc")
        assert result == "scaffold.mdc"

    def test_validate_returns_list(self):
        """validate_files() should return a list."""
        from promptosaurus.registry import registry

        result = registry.validate_files()
        assert isinstance(result, list)

    def test_validate_with_valid_files_returns_empty(self):
        """validate_files() should return empty list when all files exist."""
        from promptosaurus.registry import registry

        errors = registry.validate_files()
        # Filter out orphan warnings - we may have extra files
        missing_errors = [e for e in errors if "MISSING" in e]
        assert len(missing_errors) == 0, f"Missing files: {missing_errors}"



# PHP test framework question

from promptosaurus.questions.base.question import Question


class PhpTestFrameworkQuestion(Question):
    """Question handler for PHP test framework."""

    @property
    def key(self) -> str:
        return "php_test_framework"

    @property
    def question_text(self) -> str:
        return "What test framework do you want to use for PHP?"

    @property
    def explanation(self) -> str:
        return """Test frameworks provide structure for writing and running tests.

- PHPUnit is the industry standard with extensive features
- Pest is a modern alternative with elegant syntax built on PHPUnit"""

    @property
    def options(self) -> list[str]:
        return ["phpunit", "pest"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "phpunit": "Industry standard - xUnit style, extensive assertions, widely used",
            "pest": "Modern elegant - beautiful syntax, built on PHPUnit, growing popularity",
        }

    @property
    def default(self) -> str:
        return "phpunit"



# PHP version question

from promptosaurus.questions.base.question import Question


class PhpVersionQuestion(Question):
    """Question handler for PHP version."""

    @property
    def key(self) -> str:
        return "php_version"

    @property
    def question_text(self) -> str:
        return "What PHP version do you want to use?"

    @property
    def explanation(self) -> str:
        return """PHP version affects performance, security, and available features.

- PHP 8.x has major performance improvements and new features
- JIT compilation was introduced in PHP 8.0
- Named arguments, match expressions, and union types are 8.0+"""

    @property
    def options(self) -> list[str]:
        return ["8.3", "8.2", "8.1", "8.0"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "8.3": "Latest stable - typed class constants, dynamic class const fetch, recommended",
            "8.2": "Recent stable - readonly classes, sensitive parameter redaction",
            "8.1": "Stable - enums, readonly properties, first-class callable syntax",
            "8.0": "Major release - JIT compiler, named arguments, union types",
        }

    @property
    def default(self) -> str:
        return "8.3"



"""Builder for GitHub Copilot instruction files and ignore files.

This module provides the CopilotBuilder class that generates the configuration
files required by GitHub Copilot AI assistant.

Output:
  {output}/.github/copilot-instructions.md         ← always-on rules
  {output}/.github/instructions/{mode}.instructions.md  ← per-mode with applyTo
  {output}/.copilotignore                         ← ignore patterns
  {output}/.gitignore                             ← standard git ignore patterns

Classes:
    CopilotBuilder: Generates GitHub Copilot instruction files.

Example:
    >>> from pathlib import Path
    >>> from promptosaurus.builders.copilot import CopilotBuilder
    >>> builder = CopilotBuilder()
    >>> actions = builder.build(Path("./output"))
    >>> for action in actions[:3]:
    ...     print(action)
    ✓ .github/copilot-instructions.md
    ✓ .github/instructions/architect.instructions.md
    ✓ .github/instructions/code.instructions.md
"""

from datetime import datetime
from pathlib import Path
from typing import Any

from promptosaurus.builders.builder import Builder
from promptosaurus.builders.ignore_generator import CopilotIgnoreBuilder, GitIgnoreBuilder
from promptosaurus.registry import registry


class CopilotBuilder(Builder):
    """Builder for GitHub Copilot instruction files.

    This builder creates multiple output files for GitHub Copilot:
    - .github/copilot-instructions.md: Always-on rules concatenated
    - .github/instructions/{mode}.instructions.md: Per-mode rules with applyTo
    - .copilotignore: Ignore patterns for Copilot
    - .gitignore: Standard git ignore patterns

    GitHub Copilot uses YAML frontmatter with 'applyTo' field to determine
    which instructions apply to which files.

    Attributes:
        Inherits _base_files from Builder base class.

    Example:
        >>> builder = CopilotBuilder()
        >>> # Build all files
        >>> actions = builder.build(Path("./my-project"))
        >>> print(f\"Generated {len(actions)} files\")
    """

    def build(
        self, output: Path, config: dict[str, Any] | None = None, dry_run: bool = False
    ) -> list[str]:
        """Write Copilot instruction files under `output`.

        Generates Copilot configuration by:
        1. Creating copilot-instructions.md with always-on rules
        2. Creating per-mode instruction files with applyTo frontmatter
        3. Creating .copilotignore file
        4. Creating .gitignore file

        Args:
            output: Directory path where the files will be created.
            config: Optional configuration dict with template variables (unused for Copilot).
            dry_run: If True, preview what would be written without touching filesystem.

        Returns:
            List of action strings describing what was created.

        Example:
            >>> from pathlib import Path
            >>> builder = CopilotBuilder()
            >>> # Normal run
            >>> actions = builder.build(Path("./output"))
            >>> # Dry run
            >>> actions = builder.build(Path("./output"), dry_run=True)
        """
        actions: list[str] = []
        github = output / ".github"

        # Always-on: copilot-instructions.md
        actions.extend(self._build_always_on(github, dry_run))

        # Per-mode instruction files
        for mode_key, files in registry.mode_files.items():
            actions.extend(self._build_mode(github, mode_key, files, dry_run))

        # Build .copilotignore
        actions.extend(self._build_copilotignore(output, dry_run))

        # Build .gitignore
        actions.extend(self._build_gitignore(output, dry_run))

        return actions

    def _build_always_on(self, github: Path, dry_run: bool) -> list[str]:
        """Build the always-on copilot-instructions.md file.

        Creates the main Copilot instructions file that contains all
        always-on rules concatenated together.

        Args:
            github: The .github directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the file.
        """
        destination = github / "copilot-instructions.md"
        now = datetime.now().strftime("%Y-%m-%d %H:%M")
        lines: list[str] = [
            "# GitHub Copilot Instructions",
            "# Auto-generated by prompt CLI — edit files in prompts/ then rebuild",
            f"# Last built: {now}",
            "",
        ]
        for filename in registry.always_on:
            try:
                body = registry.prompt_body(filename)
            except FileNotFoundError:
                lines.append(f"<!-- MISSING: {filename} -->")
                continue
            lines.append("---")
            lines.append(body.rstrip())
            lines.append("")

        content = "\n".join(lines) + "\n"

        if dry_run:
            return ["[dry-run] .github/copilot-instructions.md"]

        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(content, encoding="utf-8")
        return ["✓ .github/copilot-instructions.md"]

    def _build_mode(
        self,
        github: Path,
        mode_key: str,
        files: list[str],
        dry_run: bool,
    ) -> list[str]:
        """Build a per-mode instruction file.

        Creates a mode-specific instruction file with YAML frontmatter
        containing the applyTo field for file targeting.

        Args:
            github: The .github directory path.
            mode_key: The mode identifier (e.g., 'code', 'architect').
            files: List of prompt filenames for this mode.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the file.

        Example:
            >>> actions = self._build_mode(Path(".github"), "code", ["code-feature.md"], False)
            >>> print(actions)
            ['✓ .github/instructions/code.instructions.md']
        """
        destination = github / "instructions" / f"{mode_key}.instructions.md"
        label = registry.modes.get(mode_key, mode_key.title())
        apply = registry.copilot_apply.get(mode_key, "**")

        lines: list[str] = [
            "---",
            f"applyTo: '{apply}'",
            "---",
            f"# {label} Mode Instructions",
            "",
        ]
        for filename in files:
            try:
                body = registry.prompt_body(filename)
            except FileNotFoundError:
                lines.append(f"<!-- MISSING: {filename} -->")
                continue
            lines.append(body.rstrip())
            lines.append("")

        content = "\n".join(lines) + "\n"

        if dry_run:
            return [f"[dry-run] .github/instructions/{mode_key}.instructions.md"]

        destination.parent.mkdir(parents=True, exist_ok=True)
        destination.write_text(content, encoding="utf-8")
        return [f"✓ .github/instructions/{mode_key}.instructions.md"]

    def _build_copilotignore(self, output: Path, dry_run: bool) -> list[str]:
        """Generate .copilotignore file.

        Helper method that uses CopilotIgnoreBuilder to generate the
        ignore file for GitHub Copilot.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the ignore file.
        """
        return CopilotIgnoreBuilder().build(output, dry_run)

    def _build_gitignore(self, output: Path, dry_run: bool) -> list[str]:
        """Generate .gitignore file.

        Helper method that uses GitIgnoreBuilder to generate a
        standard .gitignore file.

        Args:
            output: Output directory path.
            dry_run: If True, return preview without writing.

        Returns:
            List containing action string for the gitignore file.
        """
        return GitIgnoreBuilder().build(output, dry_run)



# Rust linter question

from promptosaurus.questions.base.question import Question


class RustLinterQuestion(Question):
    """Question for Rust linter - supports multiple selection."""

    @property
    def key(self) -> str:
        return "rust_linter"

    @property
    def question_text(self) -> str:
        return "What linter do you want to use for Rust?"

    @property
    def explanation(self) -> str:
        return """Linters check code quality, style, and potential bugs. You can select multiple:
- clippy: Official Rust linter with 500+ lints, catches common mistakes and style issues
- rustc: The Rust compiler's built-in warnings and lints, always enabled"""

    @property
    def options(self) -> list[str]:
        return ["clippy", "rustc"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "clippy": "Official linter - 500+ lints, catches common mistakes, highly recommended",
            "rustc": "Compiler lints - built-in warnings, always runs during compilation",
        }

    @property
    def default(self) -> str:
        return "clippy"

    @property
    def allow_multiple(self) -> bool:
        """Allow selecting multiple linters."""
        return True



# Dart version question

from promptosaurus.questions.base.question import Question


class DartVersionQuestion(Question):
    """Question handler for Dart version."""

    @property
    def key(self) -> str:
        return "dart_version"

    @property
    def question_text(self) -> str:
        return "What Dart version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Dart version affects language features and Flutter compatibility.

- Newer versions have improved performance and language features
- Dart powers Flutter for cross-platform mobile and web development
- Version affects available libraries and tooling"""

    @property
    def options(self) -> list[str]:
        return ["3.2", "3.1", "3.0"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "3.2": "Latest stable - best features, recommended for new projects",
            "3.1": "Recent stable - excellent all-around",
            "3.0": "Stable - widely supported, maximum compatibility",
        }

    @property
    def default(self) -> str:
        return "3.2"



# Java linter question

from promptosaurus.questions.base.question import Question


class JavaLinterQuestion(Question):
    """Question for Java linter - supports multiple selection."""

    @property
    def key(self) -> str:
        return "java_linter"

    @property
    def question_text(self) -> str:
        return "What linter(s) do you want to use?"

    @property
    def explanation(self) -> str:
        return """Linters check code quality, style, and bugs. You can select multiple:
- checkstyle: Enforces coding standards and style conventions
- spotbugs: Static analysis for bug patterns (successor to FindBugs)
- pmd: Source code analyzer for bugs, dead code, and optimizations"""

    @property
    def options(self) -> list[str]:
        return ["checkstyle", "spotbugs", "pmd"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "checkstyle": "Style enforcer - ensures code follows style conventions, highly configurable",
            "spotbugs": "Bug finder - static analysis for bug patterns, successor to FindBugs",
            "pmd": "Code analyzer - finds bugs, dead code, suboptimal code, and style issues",
        }

    @property
    def default(self) -> str:
        return "checkstyle"

    @property
    def allow_multiple(self) -> bool:
        """Allow selecting multiple linters."""
        return True



# Java test framework question

from promptosaurus.questions.base.question import Question


class JavaTestFrameworkQuestion(Question):
    """Question for Java test framework selection."""

    @property
    def key(self) -> str:
        return "java_test_framework"

    @property
    def question_text(self) -> str:
        return "What testing framework do you want to use?"

    @property
    def explanation(self) -> str:
        return """Test framework affects how tests are written and organized:
- junit5: Modern, extensible, parameterized tests, modern Java features
- junit4: Classic, stable, widely used, mature ecosystem
- testng: Flexible, powerful annotations, parallel execution, data providers"""

    @property
    def options(self) -> list[str]:
        return ["junit5", "junit4", "testng"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "junit5": "Modern standard - extensible architecture, parameterized tests, modern Java",
            "junit4": "Classic - stable, mature ecosystem, annotations-based, widely adopted",
            "testng": "Flexible - powerful data providers, parallel execution, configuration options",
        }

    @property
    def default(self) -> str:
        return "junit5"



# Rust formatter question

from promptosaurus.questions.base.question import Question


class RustFormatterQuestion(Question):
    """Question for Rust formatter."""

    @property
    def key(self) -> str:
        return "rust_formatter"

    @property
    def question_text(self) -> str:
        return "What formatter do you want to use for Rust?"

    @property
    def explanation(self) -> str:
        return """Formatters ensure consistent code style across your codebase.

- rustfmt is the official Rust formatter
- It automatically reformats code to follow Rust style guidelines
- Can be configured via rustfmt.toml for project-specific preferences
- Integrated with Cargo via `cargo fmt`"""

    @property
    def options(self) -> list[str]:
        return ["rustfmt"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "rustfmt": "Official formatter - enforces consistent style, configurable via rustfmt.toml",
        }

    @property
    def default(self) -> str:
        return "rustfmt"



"""Base selection state implementation.

This module defines the base SelectionState class using the Strategy pattern.
Different selection behaviors (single, multi, mutual exclusion) are implemented
as separate subclasses.

Classes:
    SelectionState: Abstract base class for selection state - Strategy pattern.

Example:
    >>> from promptosaurus.ui.state.selection_state import SelectionState
    >>> from promptosaurus.ui.state.single_selection_state import SingleSelectionState
    >>>
    >>> state = SingleSelectionState(selected=0, max_index=3)
    >>> print(state.current_selection)
    0
"""

from __future__ import annotations


class SelectionState:
    """Base class for selection state - Strategy pattern.

    This abstract base class defines the interface for selection state
    management. The Strategy pattern allows different selection behaviors
    to be swapped at runtime.

    Subclasses must override all methods to provide concrete implementations
    for their specific selection behavior.

    Attributes:
        current_selection: Property returning current selection(s).

    Methods:
        select: Return new state after selection (immutable).
        navigate: Return new state after navigation.
        is_selected: Check if index is selected.

    Example:
        >>> # Available implementations:
        >>> from promptosaurus.ui.state.single_selection_state import SingleSelectionState
        >>> from promptosaurus.ui.state.multi_selection_state import MultiSelectionState
        >>> from promptosaurus.ui.state.mutual_exclusion_multi_selection_state import MutualExclusionMultiSelectionState
    """

    @property
    def current_selection(self) -> int | set[int]:
        """Get current selection(s).

        Returns:
            For single selection: the selected index as int.
            For multi selection: set of selected indices.

        Raises:
            NotImplementedError: If subclass doesn't implement this property.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement current_selection")

    def select(self, index: int) -> SelectionState:
        """Return new state after selection - immutable.

        Creates a new state instance with the selection applied.
        The original state remains unchanged (immutable pattern).

        Args:
            index: The index to select.

        Returns:
            New SelectionState with the selection applied.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement select")

    def navigate(self, direction: int) -> SelectionState:
        """Return new state after navigation.

        Args:
            direction: Navigation direction (-1 for up, +1 for down).

        Returns:
            New SelectionState after navigation.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement navigate")

    def is_selected(self, index: int) -> bool:
        """Check if index is selected.

        Args:
            index: The index to check.

        Returns:
            True if the index is currently selected.

        Raises:
            NotImplementedError: If subclass doesn't implement this method.
        """
        raise NotImplementedError(f"{self.__class__.__name__} must implement is_selected")



"""Folder specification for multi-language monorepo configuration."""

from dataclasses import dataclass, field
from typing import Any

# Language-specific defaults
LANGUAGE_DEFAULTS: dict[str, dict[str, str]] = {
    "python": {
        "runtime": "3.12",
        "package_manager": "poetry",
        "test_framework": "pytest",
        "linter": "ruff",
        "formatter": "ruff",
    },
    "typescript": {
        "runtime": "5.4",
        "package_manager": "npm",
        "test_framework": "vitest",
        "linter": "eslint",
        "formatter": "prettier",
    },
    "javascript": {
        "runtime": "5.4",
        "package_manager": "npm",
        "test_framework": "vitest",
        "linter": "eslint",
        "formatter": "prettier",
    },
    "go": {
        "runtime": "1.21",
        "package_manager": "go mod",
        "test_framework": "go test",
        "linter": "golangci-lint",
        "formatter": "gofmt",
    },
    "java": {
        "runtime": "21",
        "package_manager": "maven",
        "test_framework": "junit",
        "linter": "checkstyle",
        "formatter": "google-java-format",
    },
    "rust": {
        "runtime": "1.75",
        "package_manager": "cargo",
        "test_framework": "cargo test",
        "linter": "clippy",
        "formatter": "rustfmt",
    },
    "csharp": {
        "runtime": "8.0",
        "package_manager": "nuget",
        "test_framework": "xunit",
        "linter": "roslynator",
        "formatter": "dotnet format",
    },
}


# Default coverage targets
DEFAULT_COVERAGE = {
    "line": 80,
    "branch": 70,
    "function": 90,
    "statement": 85,
    "mutation": 80,
    "path": 60,
}


@dataclass
class FolderSpec:
    """Represents configuration for a single folder in a multi-language monorepo.

    Attributes:
        folder: The folder path (e.g., "frontend", "backend", "services/auth/api")
        type: The folder type - "backend" or "frontend"
        subtype: The folder subtype:
            - backend: api, library, worker, cli
            - frontend: ui, library, e2e
        language: The programming language for this folder
        runtime: The runtime version
        package_manager: The package manager
        test_framework: The testing framework
        linter: The linter tool
        formatter: The formatter tool
        coverage: Coverage targets

    Example:
        >>> spec = FolderSpec(
        ...     folder="frontend",
        ...     type="frontend",
        ...     subtype="ui",
        ...     language="typescript",
        ... )
        >>> spec.package_manager
        'npm'
    """

    folder: str
    type: str  # "backend" or "frontend"
    subtype: str  # "api", "library", "worker", "cli" or "ui", "library", "e2e"
    language: str = ""  # Can be empty - will be derived from preset if type/subtype provided
    runtime: str = ""
    package_manager: str = ""
    test_framework: str = ""
    linter: str = ""
    formatter: str = ""
    coverage: dict[str, int] = field(default_factory=lambda: DEFAULT_COVERAGE.copy())

    def __post_init__(self) -> None:
        """Apply language-specific defaults after initialization."""
        # If language is not provided, try to derive from preset
        if not self.language and self.type and self.subtype:
            preset_defaults = get_preset_defaults(self.type, self.subtype)
            if preset_defaults and "language" in preset_defaults:
                self.language = preset_defaults["language"]

        # If still no language, use python as fallback
        if not self.language:
            self.language = "python"

        lang_key = self.language.lower()

        # Get defaults for this language
        defaults: dict[str, str] = (
            LANGUAGE_DEFAULTS.get(lang_key) or LANGUAGE_DEFAULTS.get("python") or {}
        )

        # Apply defaults if not specified
        if not self.runtime and "runtime" in defaults:
            self.runtime = defaults["runtime"]
        if not self.package_manager:
            self.package_manager = defaults.get("package_manager", "")
        if not self.test_framework:
            self.test_framework = defaults.get("test_framework", "")
        if not self.linter:
            self.linter = defaults.get("linter", "")
        if not self.formatter:
            self.formatter = defaults.get("formatter", "")

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary.

        Returns:
            Dictionary representation of the folder spec.

        Example:
            >>> spec = FolderSpec(
            ...     folder="frontend",
            ...     type="frontend",
            ...     subtype="ui",
            ...     language="typescript",
            ... )
            >>> spec.to_dict()["folder"]
            'frontend'
        """
        return {
            "folder": self.folder,
            "type": self.type,
            "subtype": self.subtype,
            "language": self.language,
            "runtime": self.runtime,
            "package_manager": self.package_manager,
            "test_framework": self.test_framework,
            "linter": self.linter,
            "formatter": self.formatter,
            "coverage": self.coverage.copy(),
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "FolderSpec":
        """Create from dictionary.

        Args:
            data: Dictionary containing folder spec data.

        Returns:
            FolderSpec instance.

        Example:
            >>> data = {
            ...     "folder": "backend",
            ...     "type": "backend",
            ...     "subtype": "api",
            ...     "language": "python",
            ... }
            >>> spec = FolderSpec.from_dict(data)
            >>> spec.folder
            'backend'
        """
        # Extract coverage if present
        coverage = data.pop("coverage", None)

        # Create instance
        instance = cls(**data)

        # Apply coverage if provided
        if coverage:
            instance.coverage = coverage

        return instance


# Standard folder type presets
FOLDER_TYPE_PRESETS = {
    "backend": {
        "api": {"language": "python", "subtype": "api"},
        "library": {"language": "python", "subtype": "library"},
        "worker": {"language": "python", "subtype": "worker"},
        "cli": {"language": "python", "subtype": "cli"},
    },
    "frontend": {
        "ui": {"language": "typescript", "subtype": "ui"},
        "library": {"language": "typescript", "subtype": "library"},
        "e2e": {"language": "typescript", "subtype": "e2e"},
    },
}


def get_preset_defaults(folder_type: str, subtype: str) -> dict[str, str]:
    """Get default values for a preset.

    Args:
        folder_type: The folder type ("backend" or "frontend")
        subtype: The folder subtype

    Returns:
        Dictionary with default values for the preset

    Example:
        >>> get_preset_defaults("backend", "api")
        {'language': 'python', 'subtype': 'api'}
    """
    return FOLDER_TYPE_PRESETS.get(folder_type, {}).get(subtype, {})



# TypeScript framework question

from promptosaurus.questions.base.question import Question


class TypeScriptFrameworkQuestion(Question):
    """Question for TypeScript framework (React, Vue, etc)."""

    @property
    def key(self) -> str:
        return "typescript_framework"

    @property
    def question_text(self) -> str:
        return "What frontend framework are you using?"

    @property
    def explanation(self) -> str:
        return """Frontend framework affects:
- Component structure
- State management patterns
- Build configuration
- Type definitions needed"""

    @property
    def options(self) -> list[str]:
        return ["none", "react", "vue", "svelte", "angular", "nextjs", "nuxt"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "none": "Vanilla TypeScript - no framework, just JS/TS",
            "react": "Meta - most popular, huge ecosystem, flexible",
            "vue": "Evan You - approachable, progressive, great docs",
            "svelte": "Rich Harris - compile-time, less boilerplate",
            "angular": "Google - enterprise, TypeScript-first, full framework",
            "nextjs": "Vercel - React meta-framework, SSR/SSG",
            "nuxt": "Vue meta-framework - SSR/SSG for Vue",
        }

    @property
    def default(self) -> str:
        return "none"



"""Tests for UI factory."""

import pytest
from sweet_tea.abstract_factory import AbstractFactory
from sweet_tea.sweet_tea_error import SweetTeaError

from promptosaurus.ui.domain.context import PipelineContext, QuestionContext
from promptosaurus.ui.domain.renderer import Renderer
from promptosaurus.ui.render.columns import ColumnLayoutRenderer
from promptosaurus.ui.render.explain import ExplainRenderer
from promptosaurus.ui.render.vertical import VerticalLayoutRenderer
from promptosaurus.ui.state.single_selection_state import SingleSelectionState
from promptosaurus.ui.ui_factory import UIFactory


class TestUIFactory:
    """Tests for UIFactory."""

    def test_create_renderer_returns_renderer_subclass(self):
        """Factory should return instances of Renderer base class."""
        question = QuestionContext(
            question="Test?",
            options=["a", "b"],
            explanations={},
            question_explanation="",
        )
        state = SingleSelectionState(selected=0, max_index=1)
        context = PipelineContext(question=question, state=state, mode="select")

        renderer = UIFactory.create_renderer(context)

        assert isinstance(renderer, Renderer)

    def test_create_renderer_selects_vertical_for_few_options(self):
        """Factory should return VerticalLayoutRenderer for <= 8 options."""
        question = QuestionContext(
            question="Test?",
            options=[str(i) for i in range(5)],
            explanations={},
            question_explanation="",
        )
        state = SingleSelectionState(selected=0, max_index=4)
        context = PipelineContext(question=question, state=state, mode="select")

        renderer = UIFactory.create_renderer(context)

        assert isinstance(renderer, VerticalLayoutRenderer)

    def test_create_renderer_selects_column_for_many_options(self):
        """Factory should return ColumnLayoutRenderer for > 8 options."""
        question = QuestionContext(
            question="Test?",
            options=[str(i) for i in range(10)],
            explanations={},
            question_explanation="",
        )
        state = SingleSelectionState(selected=0, max_index=9)
        context = PipelineContext(question=question, state=state, mode="select")

        renderer = UIFactory.create_renderer(context)

        assert isinstance(renderer, ColumnLayoutRenderer)

    def test_create_renderer_selects_explain_for_explain_mode(self):
        """Factory should return ExplainRenderer for explain mode."""
        question = QuestionContext(
            question="Test?",
            options=["a", "b"],
            explanations={},
            question_explanation="Test explanation",
        )
        state = SingleSelectionState(selected=0, max_index=1)
        context = PipelineContext(question=question, state=state, mode="explain")

        renderer = UIFactory.create_renderer(context)

        assert isinstance(renderer, ExplainRenderer)


class TestRendererRegistration:
    """Tests that renderers are properly registered with sweet_tea."""

    def test_column_layout_renderer_registered(self):
        """ColumnLayoutRenderer should be registered with snake_case key."""
        factory = AbstractFactory[Renderer]
        renderer = factory.create("column_layout_renderer")
        assert isinstance(renderer, ColumnLayoutRenderer)

    def test_vertical_layout_renderer_registered(self):
        """VerticalLayoutRenderer should be registered with snake_case key."""
        factory = AbstractFactory[Renderer]
        renderer = factory.create("vertical_layout_renderer")
        assert isinstance(renderer, VerticalLayoutRenderer)

    def test_explain_renderer_registered(self):
        """ExplainRenderer should be registered with snake_case key."""
        factory = AbstractFactory[Renderer]
        renderer = factory.create("explain_renderer")
        assert isinstance(renderer, ExplainRenderer)

    def test_unregistered_key_raises_error(self):
        """Accessing unregistered key should raise SweetTeaError."""
        factory = AbstractFactory[Renderer]
        with pytest.raises(SweetTeaError):
            factory.create("nonexistent_renderer")



"""Slow tests for promptosaurus - placeholder for future tests."""

import pytest


@pytest.mark.slow
@pytest.mark.placeholder
def test_slow_placeholder():
    """Placeholder slow test."""
    # TODO: Add slow tests when we have:
    # - Performance benchmarks
    # - Large file processing tests
    # - Stress tests
    pass



# Rust package manager question

from promptosaurus.questions.base.question import Question


class RustPackageManagerQuestion(Question):
    """Question for Rust package manager."""

    @property
    def key(self) -> str:
        return "rust_package_manager"

    @property
    def question_text(self) -> str:
        return "What package manager do you want to use for Rust?"

    @property
    def explanation(self) -> str:
        return """Package manager affects dependency resolution, build configuration, and workspace management.

- Cargo is the official and only widely-used package manager for Rust
- It handles dependency resolution, building, testing, and publishing to crates.io
- Cargo.toml defines project metadata and dependencies
- Cargo.lock ensures reproducible builds"""

    @property
    def options(self) -> list[str]:
        return ["cargo"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "cargo": "Official Rust package manager - handles dependencies, builds, tests, and publishing",
        }

    @property
    def default(self) -> str:
        return "cargo"



# Python mocking library question

from promptosaurus.questions.base.question import Question


class PythonMockingLibraryQuestion(Question):
    """Question for Python mocking library selection (multi-select)."""

    @property
    def key(self) -> str:
        return "mocking_library"

    @property
    def question_text(self) -> str:
        return "What mocking libraries do you want to use?"

    @property
    def explanation(self) -> str:
        return """Select mocking libraries for your project. Choose multiple options or select 'none' for no mocking.
- unittest.mock: Built-in, no dependencies, standard library
- pytest-mock: pytest plugin wrapping unittest.mock with convenient fixtures
- freezegun: Mock time/date for testing time-dependent code
- responses: Mock HTTP requests/responses
- none: No mocking (use real objects for integration/acceptance tests)"""

    @property
    def options(self) -> list[str]:
        return ["unittest.mock", "pytest-mock", "freezegun", "responses", "none"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "unittest.mock": "Built-in - no dependencies, standard library, widely used",
            "pytest-mock": "pytest plugin - convenient fixture injection, cleaner syntax (recommended)",
            "freezegun": "Mock time/date - freeze datetime for testing time-dependent code",
            "responses": "Mock HTTP - mock requests library responses for API testing",
            "none": "No mocking - use real objects for integration/acceptance tests",
        }

    @property
    def allow_multiple(self) -> bool:
        return True

    @property
    def default(self) -> str:
        return "pytest-mock"

    @property
    def default_indices(self) -> set[int]:
        # Default to pytest-mock (index 1)
        return {1}

    @property
    def none_index(self) -> int | None:
        # 'none' is at index 4
        return 4



# HTML formatter question

from promptosaurus.questions.base.question import Question


class HtmlFormatterQuestion(Question):
    """Question handler for HTML formatter/linter choice."""

    @property
    def key(self) -> str:
        return "html_formatter"

    @property
    def question_text(self) -> str:
        return "Which HTML formatter do you want to use?"

    @property
    def explanation(self) -> str:
        return """HTML formatters ensure consistent code style and catch common errors.

- Prettier is the most popular, handles HTML + embedded CSS/JS
- html-beautify is lightweight and HTML-focused
- djLint adds template language support (Django, Jinja2, etc.)"""

    @property
    def options(self) -> list[str]:
        return ["prettier", "html-beautify (js-beautify)", "djLint", "none"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "prettier": "Most popular - handles HTML, CSS, JS in one tool",
            "html-beautify (js-beautify)": "Lightweight HTML-focused formatter",
            "djLint": "Best for template files (Django, Jinja2, Handlebars)",
            "none": "No formatter - manual formatting only",
        }

    @property
    def default(self) -> str:
        return "prettier"



"""Unit tests for CopilotBuilder."""

import tempfile
import unittest
from pathlib import Path

from promptosaurus.builders.copilot import CopilotBuilder


class TestCopilotBuilder(unittest.TestCase):
    """Test cases for CopilotBuilder."""

    def test_copilot_builder_is_builder_subclass(self):
        """CopilotBuilder should be a subclass of Builder."""
        from promptosaurus.builders.builder import Builder

        self.assertTrue(issubclass(CopilotBuilder, Builder))

    def test_copilot_builder_has_build_method(self):
        """CopilotBuilder should have a build method."""
        builder = CopilotBuilder()
        self.assertTrue(hasattr(builder, "build"))
        self.assertTrue(callable(builder.build))

    def test_copilot_builder_build_returns_list(self):
        """build() should return a list of action strings."""
        builder = CopilotBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output)
        self.assertIsInstance(result, list)

    def test_copilot_builder_dry_run_returns_dry_run_prefix(self):
        """build() with dry_run=True should return dry run message."""
        builder = CopilotBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output, dry_run=True)
        self.assertIsInstance(result, list)

    def test_copilot_builder_returns_action_strings(self):
        """build() should return action strings."""
        builder = CopilotBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output)
        self.assertIsInstance(result, list)
        # Should have action strings
        self.assertTrue(len(result) > 0)



"""Unit tests for promptosaurus.builders.kilo."""

import tempfile
import unittest
from pathlib import Path

from promptosaurus.builders.kilo.kilo_cli import KiloCLIBuilder
from promptosaurus.builders.kilo.kilo_code_builder import KiloCodeBuilder
from promptosaurus.builders.kilo.kilo_ide import KiloIDEBuilder


class TestKiloCodeBuilderBase(unittest.TestCase):
    """Tests for KiloCodeBuilder base class methods using concrete implementation."""

    def test_language_file_map(self):
        """KiloCodeBuilder should have language_file_map property loaded from YAML."""

        # Test via instance to ensure YAML is loaded
        builder = KiloCLIBuilder()
        assert hasattr(KiloCodeBuilder, "language_file_map")
        assert "python" in builder.language_file_map
        assert builder.language_file_map["python"] == "agents/core/core-conventions-python.md"
        assert builder.language_file_map["typescript"] == "agents/core/core-conventions-typescript.md"

    def test_substitute_template_variables_basic(self):
        """KiloCodeBuilder should substitute basic template variables."""
        builder = KiloCLIBuilder()
        content = "Language: {{LANGUAGE}}, Runtime: {{RUNTIME}}"
        config = {
            "spec": {
                "language": "python",
                "runtime": "CPython",
            }
        }
        result = builder._substitute_template_variables(content, config)
        assert "python" in result
        assert "CPython" in result

    def test_substitute_template_variables_list(self):
        """KiloCodeBuilder should handle list values in templates."""
        builder = KiloCLIBuilder()
        content = "Tools: {{LINTER}}, {{FORMATTER}}"
        config = {
            "spec": {
                "linter": ["ruff", "mypy"],
                "formatter": ["ruff"],
            }
        }
        result = builder._substitute_template_variables(content, config)
        assert "ruff" in result
        assert "mypy" in result

    def test_substitute_template_variables_coverage(self):
        """KiloCodeBuilder should substitute coverage variables."""
        builder = KiloCLIBuilder()
        content = "Line: {{LINE_COVERAGE_%}}, Branch: {{BRANCH_COVERAGE_%}}"
        config = {
            "spec": {
                "coverage": {
                    "line": 90,
                    "branch": 80,
                }
            }
        }
        result = builder._substitute_template_variables(content, config)
        assert "90" in result
        assert "80" in result

    def test_substitute_template_variables_none_values(self):
        """KiloCodeBuilder should handle None values gracefully."""
        builder = KiloCLIBuilder()
        content = "Value: {{MISSING}}"
        config = {"spec": {}}
        result = builder._substitute_template_variables(content, config)
        assert "" in result or result == content


class TestKiloBuilder(unittest.TestCase):
    """Tests for KiloBuilder (now KiloCLIBuilder)."""

    def test_kilo_builder_is_builder_subclass(self):
        """KiloBuilder should be a subclass of Builder."""
        from promptosaurus.builders.builder import Builder

        assert issubclass(KiloCLIBuilder, Builder)

    def test_kilo_builder_has_build_method(self):
        """KiloBuilder should have a build method."""
        builder = KiloCLIBuilder()
        assert hasattr(builder, "build")
        assert callable(builder.build)

    def test_kilo_builder_build_returns_list(self):
        """KiloBuilder.build() should return a list of strings."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output)
            assert isinstance(result, list)
            assert all(isinstance(r, str) for r in result)

    def test_kilo_builder_build_creates_files(self):
        """KiloBuilder.build() should create files."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            builder.build(output, dry_run=False)
            # Should have created files in new structure
            assert (output / ".opencode").exists()
            assert (output / ".opencode" / "rules").exists()
            assert (output / "AGENTS.md").exists()
            assert (output / "opencode.json").exists()
            assert (output / ".kilocodemodes").exists()

    def test_kilo_builder_dry_run(self):
        """KiloBuilder.build() with dry_run=True should not write files."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            builder.build(output, dry_run=True)
            # No files should be created
            assert not (output / ".opencode").exists()
            assert not (output / "AGENTS.md").exists()
            assert not (output / "opencode.json").exists()

    def test_kilo_builder_returns_action_strings(self):
        """KiloBuilder.build() should return action strings."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output)
            assert len(result) > 0
            assert all(isinstance(r, str) for r in result)

    def test_kilo_builder_with_language_config(self):
        """KiloBuilder.build() should handle language config."""
        builder = KiloCLIBuilder()
        config = {
            "spec": {
                "language": "python",
                "test_framework": "pytest",
            }
        }
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output, config=config)
            assert isinstance(result, list)
            # Should have created language-specific file
            assert (output / ".opencode" / "rules" / "_base.md").exists()

    def test_kilo_builder_with_all_coverage_vars(self):
        """KiloBuilder should substitute all coverage template variables."""
        builder = KiloCLIBuilder()
        config = {
            "spec": {
                "language": "python",
                "coverage": {
                    "line": 90,
                    "branch": 80,
                    "function": 95,
                    "statement": 88,
                    "mutation": 85,
                    "path": 70,
                }
            }
        }
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output, config=config)
            assert isinstance(result, list)

    def test_kilo_builder_with_typescript(self):
        """KiloBuilder should handle TypeScript language config."""
        builder = KiloCLIBuilder()
        config = {
            "spec": {
                "language": "typescript",
            }
        }
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output, config=config)
            assert isinstance(result, list)

    def test_kilo_builder_with_javascript(self):
        """KiloBuilder should handle JavaScript language config."""
        builder = KiloCLIBuilder()
        config = {
            "spec": {
                "language": "javascript",
            }
        }
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            result = builder.build(output, config=config)
            assert isinstance(result, list)


class TestKiloCustomModes(unittest.TestCase):
    """Tests for custom modes filtering in Kilo builders."""

    def test_custom_modes_property_exists(self):
        """KiloCodeBuilder should have custom_modes property."""
        builder = KiloCLIBuilder()
        assert hasattr(builder, "custom_modes")
        assert isinstance(builder.custom_modes, list)

    def test_custom_modes_excludes_builtin_modes(self):
        """custom_modes should exclude built-in Kilo modes."""
        builder = KiloCLIBuilder()
        # Built-in modes should NOT be in custom_modes
        for mode in builder._kilo_builtin_modes:
            assert mode not in builder.custom_modes, f"{mode} should not be in custom_modes"

    def test_custom_modes_includes_custom_modes(self):
        """custom_modes should include non-built-in modes."""
        builder = KiloCLIBuilder()
        # These are custom modes that should be included
        expected_custom = {"test", "refactor", "document", "explain", "migration", "review", "security", "compliance", "enforcement", "planning"}
        for mode in expected_custom:
            assert mode in builder.custom_modes, f"{mode} should be in custom_modes"

    def test_cli_builder_uses_custom_modes(self):
        """KiloCLIBuilder should generate only custom mode files."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            builder.build(output, dry_run=False)
            # Check that built-in mode files are NOT created
            for mode in builder._kilo_builtin_modes:
                mode_file = output / ".opencode" / "rules" / f"{mode}.md"
                assert not mode_file.exists(), f"{mode}.md should NOT exist (built-in mode)"

    def test_ide_builder_uses_custom_modes(self):
        """KiloIDEBuilder should create mode directories for ALL 15 modes (not just custom modes)."""
        builder = KiloIDEBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            builder.build(output, dry_run=False)
            # IDE should create directories for ALL 15 modes
            for mode in builder.kilo_modes.keys():
                mode_dir = output / ".kilocode" / f"rules-{mode}"
                assert mode_dir.exists(), f"rules-{mode}/ should exist for IDE"


class TestKiloCLIAgentsContent(unittest.TestCase):
    """Tests for AGENTS.md content generation in KiloCLIBuilder."""

    def test_get_agents_md_content_exists(self):
        """KiloCLIBuilder should have _get_agents_md_content method."""
        builder = KiloCLIBuilder()
        assert hasattr(builder, "_get_agents_md_content")
        assert callable(builder._get_agents_md_content)

    def test_agents_md_content_includes_cli_structure(self):
        """KiloCLIBuilder AGENTS.md should include CLI structure info."""
        builder = KiloCLIBuilder()
        content = builder._get_agents_md_content()
        # Should mention .opencode/rules/ directory structure
        assert ".opencode/rules/" in content
        assert "CLI format" in content
        # Should mention _base.md
        assert "_base.md" in content

    def test_agents_md_content_includes_all_modes(self):
        """KiloCLIBuilder AGENTS.md should list all modes."""
        builder = KiloCLIBuilder()
        content = builder._get_agents_md_content()
        # Should include key modes
        assert "architect" in content
        assert "code" in content
        assert "test" in content
        assert "debug" in content

    def test_agents_md_created_with_content(self):
        """KiloCLIBuilder should create AGENTS.md with correct content."""
        builder = KiloCLIBuilder()
        with tempfile.TemporaryDirectory() as tmpdir:
            output = Path(tmpdir)
            builder.build(output, dry_run=False)
            agents_file = output / "AGENTS.md"
            assert agents_file.exists()
            content = agents_file.read_text(encoding="utf-8")
            # Verify CLI-specific content
            assert ".opencode/rules/" in content
            assert "CLI format" in content
            # Should have core instructions section
            assert "Core Instructions" in content



"""Public UI API - main entry point for interactive selection.

This module provides the primary functions for interactive command-line
selection, confirmation, and prompting. These functions are the main
public API for the promptosaurus UI system.

Functions:
    select_option_with_explain: Interactive selection with number keys and explain option.
    confirm_interactive: Yes/no confirmation dialog.
    prompt_with_default: Input prompt with default value.

Example:
    >>> from promptosaurus.ui import select_option_with_explain, confirm_interactive
    >>>
    >>> # Single selection
    >>> language = select_option_with_explain(
    ...     question="Choose a language:",
    ...     options=["Python", "TypeScript", "Go"],
    ...     explanations={
    ...         "Python": "Popular for data science and AI",
    ...         "TypeScript": "Type-safe JavaScript for web",
    ...         "Go": "Systems programming by Google"
    ...     },
    ...     question_explanation="Select your primary language"
    ... )
    >>>
    >>> # Confirmation
    >>> if confirm_interactive("Continue with installation?"):
    ...     print("Proceeding...")
"""

from promptosaurus.ui.domain.context import QuestionContext
from promptosaurus.ui.pipeline.orchestrator import PipelineOrchestrator
from promptosaurus.ui.pipeline.render_stage import RenderStage
from promptosaurus.ui.pipeline.state_update_stage import StateUpdateStage
from promptosaurus.ui.ui_factory import UIFactory


def select_option_with_explain(
    question: str,
    options: list[str],
    explanations: dict[str, str],
    question_explanation: str,
    default_index: int = 0,
    default_indices: set[int] | None = None,
    allow_multiple: bool = False,
    none_index: int | None = None,
) -> str | list[str]:
    """Interactive selection with number keys and explain option.

    This is the main entry point for interactive selection in promptosaurus.
    It presents options to the user with explanations and handles keyboard
    input for selection. The function uses the pipeline architecture
    internally but maintains backwards compatibility with existing code.

    Args:
        question: The main question/prompt to display to the user.
        options: List of available options to choose from.
        explanations: Dictionary mapping option strings to their explanations.
        question_explanation: Detailed explanation of what the question means.
        default_index: Index of the default option (used when user presses Enter).
        default_indices: Set of default indices for multi-select.
        allow_multiple: If True, allow selecting multiple options.
        none_index: Optional index for a "none of the above" option.

    Returns:
        If allow_multiple is False: The selected option string.
        If allow_multiple is True: List of selected option strings.

    Raises:
        UserCancelledError: If the user presses the quit key (typically 'q' or 'Escape').

    Example:
        >>> result = select_option_with_explain(
        ...     question="Choose a language:",
        ...     options=["Python", "TypeScript", "Go"],
        ...     explanations={
        ...         "Python": "Popular for data science",
        ...         "TypeScript": "Type-safe JavaScript",
        ...         "Go": "Systems programming"
        ...     },
        ...     question_explanation="Select your primary language",
        ...     default_index=0
        ... )
        >>> print(result)
        Python
    """
    context = QuestionContext(
        question=question,
        options=options,
        explanations=explanations,
        question_explanation=question_explanation,
        default_index=default_index,
        default_indices=default_indices if default_indices is not None else {default_index},
        allow_multiple=allow_multiple,
        none_index=none_index,
    )

    input_provider = UIFactory.create_input_provider()
    render_stage = RenderStage(renderer_selector=UIFactory.create_renderer)
    state_update = StateUpdateStage()

    pipeline = PipelineOrchestrator(
        input_provider=input_provider,
        render_stage=render_stage,
        state_update_stage=state_update,
    )

    return pipeline.run(context)


def confirm_interactive(prompt: str, default: bool = True) -> bool:
    """Yes/no confirmation dialog.

    Presents a simple yes/no confirmation to the user and returns
    their choice as a boolean.

    Args:
        prompt: The confirmation question to display.
        default: The default choice if user presses Enter (True for "Yes", False for "No").

    Returns:
        True if user confirmed ("Yes"), False if they declined ("No").

    Example:
        >>> if confirm_interactive("Install dependencies?"):
        ...     print("Installing...")
        >>> # Or with No as default
        >>> if not confirm_interactive("Delete files?", default=False):
        ...     print("Keeping files")
    """
    result = select_option_with_explain(
        question=prompt,
        options=["Yes", "No"],
        explanations={"Yes": "Confirm", "No": "Cancel"},
        question_explanation=prompt,
        default_index=0 if default else 1,
    )
    return result == "Yes"


def prompt_with_default(prompt: str, default: str) -> str:
    """Prompt with default value.

    Displays a prompt with a default value. If the user enters nothing,
    the default is returned.

    Args:
        prompt: The prompt text to display.
        default: The default value to use if input is empty.

    Returns:
        The user's input if non-empty, otherwise the default value.

    Example:
        >>> name = prompt_with_default("Enter your name", "Anonymous")
        >>> # If user presses Enter without typing: returns "Anonymous"
        >>> # If user types "John": returns "John"
    """
    suffix = f" [{default}]" if default else ""
    response = input(f"{prompt}{suffix}: ").strip()
    return response if response else default



# Question handlers for different repository types

from typing import Any


class LanguageQuestionHandler:
    """
    Base class for handling language-specific questions.

    This class defines the interface for question handlers.
    Subclasses should implement the handle() method to provide
    specific behavior for different repository types.
    """

    def handle(self, repo_type: str) -> dict[str, Any]:
        """
        Handle questions for the given repository type.

        Args:
            repo_type: The repository type string

        Returns:
            Configuration dictionary with user responses
        """
        raise NotImplementedError("Subclasses must implement handle()")



# Java version question

from promptosaurus.questions.base.question import Question


class JavaVersionQuestion(Question):
    """Question handler for Java version selection."""

    @property
    def key(self) -> str:
        return "java_version"

    @property
    def question_text(self) -> str:
        return "What Java version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Java version affects language features, performance, and library compatibility.

- Newer versions have better performance and modern language features
- LTS (Long Term Support) versions receive updates for many years
- Some libraries require minimum Java versions
- Java 21 is the latest LTS with virtual threads and improved pattern matching"""

    @property
    def options(self) -> list[str]:
        return ["21", "17", "11", "8"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "21": "Latest LTS - virtual threads, pattern matching, sequenced collections, recommended",
            "17": "Mature LTS - sealed classes, pattern matching for switch, widely adopted",
            "11": "Legacy LTS - first modular JDK, var for local variables, still widely used",
            "8": "Classic - lambdas, streams, maximum compatibility, minimal language changes",
        }

    @property
    def default(self) -> str:
        return "21"



"""Kilo Code builder implementations.

This package provides builder classes for generating Kilo Code configuration
files in both CLI and IDE formats.

Classes:
    KiloCodeBuilder: Base class for Kilo Code builders.
    KiloCLIBuilder: Builder for CLI format (.opencode/rules/).
    KiloIDEBuilder: Builder for IDE format (.kilocode/rules-{mode}/).

Functions:
    KiloConfig: Configuration loader for Kilo builder settings.

Example:
    >>> from promptosaurus.builders.kilo import KiloCLIBuilder, KiloIDEBuilder
    >>> from pathlib import Path

    >>> # CLI format
    >>> cli_builder = KiloCLIBuilder()
    >>> cli_actions = cli_builder.build(Path("./cli-output"))

    >>> # IDE format
    >>> ide_builder = KiloIDEBuilder()
    >>> ide_actions = ide_builder.build(Path("./ide-output"))
"""



"""
promptosaurus.builders
Builders for different AI assistant configurations.

Import builders directly from their modules:
    from promptosaurus.builders.kilo_cli import KiloCLIBuilder
    from promptosaurus.builders.kilo_ide import KiloIDEBuilder
    from promptosaurus.builders.kilo import KiloCodeBuilder
    from promptosaurus.builders.builder import Builder
    from promptosaurus.builders.cline import ClineBuilder
    from promptosaurus.builders.cursor import CursorBuilder
    from promptosaurus.builders.copilot import CopilotBuilder
"""

# No exports - import directly from submodules per conventions.



# Ruby package manager question

from promptosaurus.questions.base.question import Question


class RubyPackageManagerQuestion(Question):
    """Question handler for Ruby package manager."""

    @property
    def key(self) -> str:
        return "ruby_package_manager"

    @property
    def question_text(self) -> str:
        return "What package manager do you want to use for Ruby?"

    @property
    def explanation(self) -> str:
        return """Package managers handle dependency installation and versioning.

- Bundler is the standard for application dependency management
- RubyGems is the built-in package manager for installing gems"""

    @property
    def options(self) -> list[str]:
        return ["bundler", "rubygems"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "bundler": "Standard for apps - manages gem dependencies via Gemfile, recommended",
            "rubygems": "Built-in - direct gem installation, less common for projects",
        }

    @property
    def default(self) -> str:
        return "bundler"



# TypeScript package manager question

from promptosaurus.questions.base.question import Question


class TypeScriptPackageManagerQuestion(Question):
    """Question for TypeScript package manager."""

    @property
    def key(self) -> str:
        return "typescript_package_manager"

    @property
    def question_text(self) -> str:
        return "What package manager do you want to use for JavaScript/TypeScript?"

    @property
    def explanation(self) -> str:
        return """Package manager affects:
- Installation speed
- Lock file handling
- Workspace support
- Node version management"""

    @property
    def options(self) -> list[str]:
        return ["npm", "pnpm", "yarn"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "npm": "Official - largest ecosystem, good for most projects",
            "pnpm": "Fast, efficient - disk space savings, strict node_modules",
            "yarn": "Facebook - good features, widely used, npm alternative",
        }

    @property
    def default(self) -> str:
        return "npm"



# Haskell compiler question

from promptosaurus.questions.base.question import Question


class HaskellCompilerQuestion(Question):
    """Question handler for Haskell compiler."""

    @property
    def key(self) -> str:
        return "haskell_compiler"

    @property
    def question_text(self) -> str:
        return "What Haskell compiler do you want to use?"

    @property
    def explanation(self) -> str:
        return """The Haskell compiler translates Haskell code into executable programs.

- GHC (Glasgow Haskell Compiler) is the standard and most widely used compiler
- It provides advanced optimizations and language extensions
- GHC is the de facto standard for Haskell development"""

    @property
    def options(self) -> list[str]:
        return ["ghc"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "ghc": "Glasgow Haskell Compiler - the standard Haskell compiler",
        }

    @property
    def default(self) -> str:
        return "ghc"



# Terraform version question

from promptosaurus.questions.base.question import Question


class TerraformVersionQuestion(Question):
    """Question handler for Terraform version."""

    @property
    def key(self) -> str:
        return "terraform_version"

    @property
    def question_text(self) -> str:
        return "What Terraform version do you want to use?"

    @property
    def explanation(self) -> str:
        return """Terraform version affects provider compatibility and language features.

- Newer versions have improved language features and bug fixes
- Terraform manages infrastructure as code across multiple providers
- Version affects available providers and module compatibility"""

    @property
    def options(self) -> list[str]:
        return ["1.7", "1.6", "1.5"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "1.7": "Latest stable - best features, recommended for new projects",
            "1.6": "Recent stable - excellent all-around",
            "1.5": "Stable - widely tested, maximum compatibility",
        }

    @property
    def default(self) -> str:
        return "1.7"



"""Unix-specific input provider."""

from collections.abc import Iterator

from promptosaurus.ui.domain.events import InputEvent, InputEventType
from promptosaurus.ui.domain.input_provider import InputProvider


class UnixInputProvider(InputProvider):
    """Unix-specific input using termios/tty."""

    @property
    def events(self) -> Iterator[InputEvent]:
        """Yield input events."""
        import sys
        import termios
        import tty

        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)  # type: ignore[attr-defined]

        try:
            tty.setraw(fd)  # type: ignore[attr-defined]
            while True:
                key = sys.stdin.read(1)
                yield self._parse_key(key, sys.stdin)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)  # type: ignore[attr-defined]

    @staticmethod
    def _parse_key(key: str, stdin) -> InputEvent:
        """Parse Unix key codes into events."""
        if key == "\r":
            return InputEvent(event_type=InputEventType.ENTER)
        elif key == "q":
            return InputEvent(event_type=InputEventType.QUIT)
        elif key == "\x1b":  # Escape sequence (arrow keys)
            import select

            # Check if there's more data to read (arrow keys)
            if select.select([stdin], [], [], 0.05)[0]:
                seq = stdin.read(2)
                if seq == "[A":
                    return InputEvent(event_type=InputEventType.UP)
                elif seq == "[B":
                    return InputEvent(event_type=InputEventType.DOWN)
            # Just ESC key pressed - ignore it (not shown in UI)
            return InputEvent(event_type=InputEventType.UNKNOWN, raw_key=key)
        elif key == "\x03":  # Ctrl+C
            return InputEvent(event_type=InputEventType.QUIT)
        elif key.isdigit():
            return InputEvent(event_type=InputEventType.NUMBER, value=int(key))
        elif key.lower() in "abcd":
            # Map letters a-d (and A-D) to selection indices 0-3
            letter_to_index = {"a": 0, "b": 1, "c": 2, "d": 3}
            index = letter_to_index[key.lower()]
            return InputEvent(event_type=InputEventType.NUMBER, value=index)
        elif key.lower() == "e":
            # 'e' or 'E' triggers explain mode directly
            return InputEvent(event_type=InputEventType.EXPLAIN)

        return InputEvent(event_type=InputEventType.UNKNOWN, raw_key=key)

    def supports_raw(self) -> bool:
        """Whether raw input is supported."""
        return True



"""Tests for promptosaurus.questions.handlers module."""

from unittest.mock import Mock

import pytest

from promptosaurus.questions.handlers.handle_single_language_questions import (
    HandleSingleLanguageQuestions,
)
from promptosaurus.questions.handlers.language_question_handler import LanguageQuestionHandler


class TestLanguageQuestionHandler:
    """Tests for LanguageQuestionHandler base class."""

    def test_handle_method_raises_not_implemented(self):
        """Base handle() method should raise NotImplementedError."""
        handler = LanguageQuestionHandler()

        with pytest.raises(NotImplementedError):
            handler.handle("single-language")


class TestHandleSingleLanguageQuestions:
    """Tests for HandleSingleLanguageQuestions handler."""

    def test_handler_initializes_with_selector(self):
        """Handler should initialize with a selector function."""
        mock_selector = Mock(return_value="python")
        handler = HandleSingleLanguageQuestions(mock_selector)

        assert handler.select_option == mock_selector

    def test_handle_returns_config_dict(self, monkeypatch):
        """Handle should return a configuration dictionary."""
        mock_selector = Mock(return_value="python")
        handler = HandleSingleLanguageQuestions(mock_selector)

        # Mock click.echo to avoid output during test
        monkeypatch.setattr("click.echo", lambda x: None)

        config = handler.handle("single-language")

        assert isinstance(config, dict)
        assert "spec" in config

    def test_python_questions_in_config(self, monkeypatch):
        """Python questions should populate config defaults."""
        mock_selector = Mock(return_value="python")
        handler = HandleSingleLanguageQuestions(mock_selector)

        # Mock click.echo to avoid output during test
        monkeypatch.setattr("click.echo", lambda x: None)

        config = handler.handle("single-language")

        assert "spec" in config
        # Config should have some spec populated
        assert isinstance(config["spec"], dict)

    def test_handler_with_mock_selector_returning_ruff(self, monkeypatch):
        """Handler should work with selector returning python for linter."""
        # First call returns "python" for language selection
        # Subsequent calls return answers for each question
        mock_selector = Mock(return_value="python")
        handler = HandleSingleLanguageQuestions(mock_selector)

        # Mock click.echo to avoid output during test
        monkeypatch.setattr("click.echo", lambda x: None)

        config = handler.handle("single-language")

        # Verify the config was created
        assert config is not None
        assert "spec" in config



# Ruby test framework question

from promptosaurus.questions.base.question import Question


class RubyTestFrameworkQuestion(Question):
    """Question handler for Ruby test framework."""

    @property
    def key(self) -> str:
        return "ruby_test_framework"

    @property
    def question_text(self) -> str:
        return "What test framework do you want to use for Ruby?"

    @property
    def explanation(self) -> str:
        return """Test frameworks provide structure for writing and running tests.

- RSpec is behavior-driven with expressive syntax
- Minitest is simpler, built into Ruby stdlib"""

    @property
    def options(self) -> list[str]:
        return ["rspec", "minitest"]

    @property
    def option_explanations(self) -> dict[str, str]:
        return {
            "rspec": "Behavior-driven - expressive DSL, great mocking, recommended",
            "minitest": "Built-in - simpler, faster, no dependencies",
        }

    @property
    def default(self) -> str:
        return "rspec"



"""Domain events for UI input handling.

This module provides the event types and event models used for
handling keyboard input in the UI pipeline.

Classes:
    InputEventType: Enum of supported input event types.
    InputEvent: Immutable input event with type and value.

Example:
    >>> from promptosaurus.ui.domain.events import InputEventType, InputEvent
    >>>
    >>> # Create events
    >>> num_event = InputEvent(event_type=InputEventType.NUMBER, value=1, raw_key="1")
    >>> enter_event = InputEvent(event_type=InputEventType.ENTER, raw_key="\\n")
    >>>
    >>> print(num_event.event_type)
    InputEventType.NUMBER
"""

from enum import Enum, auto

from pydantic import BaseModel


class InputEventType(Enum):
    """Types of input events - extensible without code changes.

    This enum defines all supported keyboard input types. New event
    types can be added without modifying existing code.

    Attributes:
        NUMBER: Number key pressed (0-9) for option selection.
        UP: Up arrow key for navigation.
        DOWN: Down arrow key for navigation.
        ENTER: Enter key to confirm selection.
        QUIT: Quit/cancel key (q or Escape).
        EXPLAIN: Direct trigger for explain mode (e key).
        UNKNOWN: Unrecognized key press.

    Example:
        >>> print(InputEventType.UP.name)
        UP
        >>> InputEventType.UP == InputEventType.UP
        True
    """

    NUMBER = auto()
    UP = auto()
    DOWN = auto()
    ENTER = auto()
    QUIT = auto()
    EXPLAIN = auto()  # Direct trigger for explain mode
    UNKNOWN = auto()


class InputEvent(BaseModel):
    """Immutable input event - all context included.

    This Pydantic model represents a single input event from the user.
    It includes the event type, optional numeric value (for NUMBER events),
    and the raw key representation.

    Attributes:
        event_type: The type of input event.
        value: Optional integer value for NUMBER events (0-9).
        raw_key: The raw key representation as string or bytes.

    Config:
        frozen: True - instances are immutable after creation.

    Example:
        >>> event = InputEvent(event_type=InputEventType.NUMBER, value=2, raw_key="2")
        >>> print(event.value)
        2
        >>> event.event_type == InputEventType.NUMBER
        True
    """

    event_type: InputEventType
    value: int | None = None  # For NUMBER events
    raw_key: str | bytes = ""

    class Config:
        frozen = True



"""Configuration options management for the prompt update command.

This module provides data structures and functions for managing configuration
options that can be updated interactively via the `prompt update` command.

The module defines:
    - ConfigOption dataclass: Represents a single updateable configuration option
    - CONFIG_OPTIONS: List of all available options
    - Helper functions for loading, getting, and setting nested config values

Constants:
    REPO_TYPE_OPTIONS: Available repository type options
    PACKAGE_MANAGER_OPTIONS: Available package manager options
    TEST_FRAMEWORK_OPTIONS: Available test framework options
    LINTER_OPTIONS: Available linter options
    FORMATTER_OPTIONS: Available formatter options

Classes:
    ConfigOption: Dataclass representing a single configuration option

Functions:
    load_current_values: Load current config values into ConfigOption objects
    get_nested_value: Get value from nested dict using dot notation
    set_nested_value: Set value in nested dict using dot notation

Example:
    >>> from promptosaurus.config_options import CONFIG_OPTIONS, load_current_values
    >>> config = {"spec": {"language": "python"}}
    >>> options = load_current_values(config)
    >>> options[0].current_value
    'python'
"""

from dataclasses import dataclass
from typing import Any

from promptosaurus.questions.language import LANGUAGE_KEYS

# Available options for single-select config fields
REPO_TYPE_OPTIONS = ["single-language", "multi-language-monorepo", "mixed-collocation"]

PACKAGE_MANAGER_OPTIONS = [
    "poetry",
    "npm",
    "pip",
    "yarn",
    "pnpm",
    "bun",
    "cargo",
    "gradle",
    "maven",
    "dotnet",
]

TEST_FRAMEWORK_OPTIONS = [
    "pytest",
    "vitest",
    "jest",
    "go test",
    "junit",
    "rspec",
    "phpunit",
    "swift testing",
    "kotest",
    "xctest",
]

LINTER_OPTIONS = [
    "ruff",
    "eslint",
    "pylint",
    "golangci-lint",
    "checkstyle",
    "rubocop",
    "phpcs",
    "swiftlint",
    "detekt",
]

FORMATTER_OPTIONS = [
    "ruff",
    "prettier",
    "black",
    "gofmt",
    "dotnet format",
    "rubocop",
    "php-cs-fixer",
]


@dataclass
class ConfigOption:
    """Represents a single configuration option that can be updated interactively.

    This dataclass defines a configuration option that users can modify via the
    `prompt update` command. Each option has a key (used in the config file),
    a display name (shown to users), a type, and available choices if applicable.

    Attributes:
        key: The configuration key in dot notation (e.g., "spec.language").
        display_name: Human-readable name shown in the UI.
        option_type: Type of option - "single-select", "text", or "composite".
        current_value: The current value from the configuration file.
        available_options: List of valid choices for single-select options.

    Example:
        >>> opt = ConfigOption(
        ...     key="spec.language",
        ...     display_name="Language",
        ...     option_type="single-select",
        ...     available_options=["python", "typescript"]
        ... )
        >>> opt.key
        'spec.language'
    """

    key: str
    display_name: str
    option_type: str  # "single-select", "text", "composite"
    current_value: Any = None
    available_options: list[str] | None = None


# Define all updateable options (excluding AI tool which is handled by switch)
CONFIG_OPTIONS: list[ConfigOption] = [
    ConfigOption(
        key="repository.type",
        display_name="Repository Type",
        option_type="single-select",
        available_options=REPO_TYPE_OPTIONS,
    ),
    ConfigOption(
        key="spec.language",
        display_name="Language",
        option_type="single-select",
        available_options=LANGUAGE_KEYS,
    ),
    ConfigOption(
        key="spec.runtime",
        display_name="Runtime",
        option_type="text",
    ),
    ConfigOption(
        key="spec.package_manager",
        display_name="Package Manager",
        option_type="single-select",
        available_options=PACKAGE_MANAGER_OPTIONS,
    ),
    ConfigOption(
        key="spec.test_framework",
        display_name="Test Framework",
        option_type="single-select",
        available_options=TEST_FRAMEWORK_OPTIONS,
    ),
    ConfigOption(
        key="spec.linter",
        display_name="Linter",
        option_type="single-select",
        available_options=LINTER_OPTIONS,
    ),
    ConfigOption(
        key="spec.formatter",
        display_name="Formatter",
        option_type="single-select",
        available_options=FORMATTER_OPTIONS,
    ),
    ConfigOption(
        key="spec.coverage",
        display_name="Coverage Targets",
        option_type="composite",
    ),
]


def load_current_values(
    config: dict[str, Any], options: list[ConfigOption] | None = None
) -> list[ConfigOption]:
    """Load current values from config into ConfigOption objects.

    This function populates the `current_value` attribute of each ConfigOption
    by looking up values in the nested configuration dictionary using dot notation.

    Args:
        config: The configuration dictionary loaded from YAML.
        options: Optional list of ConfigOption objects. Defaults to CONFIG_OPTIONS.

    Returns:
        List of ConfigOption objects with current values populated.

    Example:
        >>> config = {"spec": {"language": "python"}}
        >>> options = load_current_values(config)
        >>> options[0].current_value
        'python'
    """
    if options is None:
        options = CONFIG_OPTIONS.copy()

    for opt in options:
        # Get value from nested config using dot notation
        value: Any = config
        for key in opt.key.split("."):
            if isinstance(value, dict):
                value = value.get(key)
            else:
                value = None
                break
        opt.current_value = value

    return options


def get_nested_value(config: dict[str, Any], key: str) -> Any:
    """Get a value from nested dict using dot notation.

    Args:
        config: The configuration dictionary
        key: Dot-separated key path (e.g., "spec.language")

    Returns:
        The value at the key path, or None if not found
    """
    value: Any = config
    for part in key.split("."):
        if isinstance(value, dict):
            value = value.get(part)
        else:
            return None
    return value


def set_nested_value(config: dict[str, Any], key: str, value: Any) -> None:
    """Set a value in nested dict using dot notation (mutates in place).

    Args:
        config: The configuration dictionary (mutated in place)
        key: Dot-separated key path (e.g., "spec.language")
        value: The value to set
    """
    parts = key.split(".")
    current = config

    # Navigate to the parent of the target key
    for part in parts[:-1]:
        if part not in current:
            current[part] = {}
        current = current[part]

    # Set the final value
    current[parts[-1]] = value
